(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined'
        ? factory(
              exports,
              require('@angular/core'),
              require('@angular/common'),
              require('rxjs'),
              require('@angular/platform-browser'),
              require('rxjs/operators'),
              require('@angular/router'),
              require('@angular/animations'),
          )
        : typeof define === 'function' && define.amd
        ? define('ngx-owl-carousel-o', [
              'exports',
              '@angular/core',
              '@angular/common',
              'rxjs',
              '@angular/platform-browser',
              'rxjs/operators',
              '@angular/router',
              '@angular/animations',
          ], factory)
        : ((global = global || self),
          factory(
              (global['ngx-owl-carousel-o'] = {}),
              global.ng.core,
              global.ng.common,
              global.rxjs,
              global.ng.platformBrowser,
              global.rxjs.operators,
              global.ng.router,
              global.ng.animations,
          ));
})(this, function (exports, core, common, rxjs, platformBrowser, operators, router, animations) {
    'use strict';

    var ResizeService = /** @class */ (function () {
        function ResizeService(eventManager) {
            this.eventManager = eventManager;
            this.resizeSubject = new rxjs.Subject();
            this.eventManager.addGlobalEventListener('window', 'resize', this.onResize.bind(this));
            this.eventManager.addGlobalEventListener('window', 'onload', this.onLoaded.bind(this));
        }
        Object.defineProperty(ResizeService.prototype, 'onResize$', {
            /**
             * Makes resizeSubject become Observable
             * @returns Observable of resizeSubject
             */
            get: function () {
                return this.resizeSubject.asObservable();
            },
            enumerable: false,
            configurable: true,
        });
        /**
         * Handler of 'resize' event. Passes data throw resizeSubject
         * @param event Event Object of 'resize' event
         */
        ResizeService.prototype.onResize = function (event) {
            this.resizeSubject.next(event.target);
        };
        /**
         * Handler of 'onload' event. Defines the width of window
         * @param event Event Object of 'onload' event
         */
        ResizeService.prototype.onLoaded = function (event) {
            this.windowWidth = event.target;
        };
        return ResizeService;
    })();
    ResizeService.decorators = [{ type: core.Injectable }];
    ResizeService.ctorParameters = function () {
        return [{ type: platformBrowser.EventManager }];
    };

    /**
     * Defaults value of options
     */
    var OwlCarouselOConfig = /** @class */ (function () {
        function OwlCarouselOConfig() {
            this.items = 3;
            this.skip_validateItems = false;
            this.loop = false;
            this.center = false;
            this.rewind = false;
            this.mouseDrag = true;
            this.touchDrag = true;
            this.pullDrag = true;
            this.freeDrag = false;
            this.margin = 0;
            this.stagePadding = 0;
            this.merge = false;
            this.mergeFit = true;
            this.autoWidth = false;
            this.startPosition = 0;
            this.rtl = false;
            this.smartSpeed = 250;
            this.fluidSpeed = false;
            this.dragEndSpeed = false;
            this.responsive = {};
            this.responsiveRefreshRate = 200;
            // defaults to Navigation
            this.nav = false;
            this.navText = ['prev', 'next'];
            this.navSpeed = false;
            this.slideBy = 1; // stage moves on 1 width of slide; if slideBy = 2, stage moves on 2 widths of slide
            this.dots = true;
            this.dotsEach = false;
            this.dotsData = false;
            this.dotsSpeed = false;
            // defaults to Autoplay
            this.autoplay = false;
            this.autoplayTimeout = 5000;
            this.autoplayHoverPause = false;
            this.autoplaySpeed = false;
            this.autoplayMouseleaveTimeout = 1;
            // defaults to LazyLoading
            this.lazyLoad = false;
            this.lazyLoadEager = 0;
            // defaults to Animate
            this.slideTransition = '';
            this.animateOut = false;
            this.animateIn = false;
            // defaults to AutoHeight
            this.autoHeight = false;
            // defaults to Hash
            this.URLhashListener = false;
        }
        return OwlCarouselOConfig;
    })();
    /**
     * we can't read types from OwlOptions in javascript because of props have undefined value and types of those props are used for validating inputs
     * class below is copy of OwlOptions but its all props have string value showing certain type;
     * this is class is being used just in method _validateOptions() of CarouselService;
     */
    var OwlOptionsMockedTypes = /** @class */ (function () {
        function OwlOptionsMockedTypes() {
            this.items = 'number';
            this.skip_validateItems = 'boolean';
            this.loop = 'boolean';
            this.center = 'boolean';
            this.rewind = 'boolean';
            this.mouseDrag = 'boolean';
            this.touchDrag = 'boolean';
            this.pullDrag = 'boolean';
            this.freeDrag = 'boolean';
            this.margin = 'number';
            this.stagePadding = 'number';
            this.merge = 'boolean';
            this.mergeFit = 'boolean';
            this.autoWidth = 'boolean';
            this.startPosition = 'number|string';
            this.rtl = 'boolean';
            this.smartSpeed = 'number';
            this.fluidSpeed = 'boolean';
            this.dragEndSpeed = 'number|boolean';
            this.responsive = {};
            this.responsiveRefreshRate = 'number';
            // defaults to Navigation
            this.nav = 'boolean';
            this.navText = 'string[]';
            this.navSpeed = 'number|boolean';
            this.slideBy = 'number|string'; // stage moves on 1 width of slide; if slideBy = 2, stage moves on 2 widths of slide
            this.dots = 'boolean';
            this.dotsEach = 'number|boolean';
            this.dotsData = 'boolean';
            this.dotsSpeed = 'number|boolean';
            // defaults to Autoplay
            this.autoplay = 'boolean';
            this.autoplayTimeout = 'number';
            this.autoplayHoverPause = 'boolean';
            this.autoplaySpeed = 'number|boolean';
            this.autoplayMouseleaveTimeout = 'number';
            // defaults to LazyLoading
            this.lazyLoad = 'boolean';
            this.lazyLoadEager = 'number';
            // defaults to Animate
            this.slideTransition = 'string';
            this.animateOut = 'string|boolean';
            this.animateIn = 'string|boolean';
            // defaults to AutoHeight
            this.autoHeight = 'boolean';
            // defaults to Hash
            this.URLhashListener = 'boolean';
        }
        return OwlOptionsMockedTypes;
    })();

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics =
            Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array &&
                function (d, b) {
                    d.__proto__ = b;
                }) ||
            function (d, b) {
                for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
            };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : ((__.prototype = b.prototype), new __());
    }
    var __assign = function () {
        __assign =
            Object.assign ||
            function __assign(t) {
                for (var s, i = 1, n = arguments.length; i < n; i++) {
                    s = arguments[i];
                    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
                }
                return t;
            };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === 'function')
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length,
            r = c < 3 ? target : desc === null ? (desc = Object.getOwnPropertyDescriptor(target, key)) : desc,
            d;
        if (typeof Reflect === 'object' && typeof Reflect.decorate === 'function') r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if ((d = decorators[i])) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) {
            decorator(target, key, paramIndex);
        };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === 'object' && typeof Reflect.metadata === 'function') return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) {
            return value instanceof P
                ? value
                : new P(function (resolve) {
                      resolve(value);
                  });
        }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) {
                try {
                    step(generator.next(value));
                } catch (e) {
                    reject(e);
                }
            }
            function rejected(value) {
                try {
                    step(generator['throw'](value));
                } catch (e) {
                    reject(e);
                }
            }
            function step(result) {
                result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
            }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = {
                label: 0,
                sent: function () {
                    if (t[0] & 1) throw t[1];
                    return t[1];
                },
                trys: [],
                ops: [],
            },
            f,
            y,
            t,
            g;
        return (
            (g = { next: verb(0), throw: verb(1), return: verb(2) }),
            typeof Symbol === 'function' &&
                (g[Symbol.iterator] = function () {
                    return this;
                }),
            g
        );
        function verb(n) {
            return function (v) {
                return step([n, v]);
            };
        }
        function step(op) {
            if (f) throw new TypeError('Generator is already executing.');
            while (_)
                try {
                    if (
                        ((f = 1),
                        y && (t = op[0] & 2 ? y['return'] : op[0] ? y['throw'] || ((t = y['return']) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                    )
                        return t;
                    if (((y = 0), t)) op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!((t = _.trys), (t = t.length > 0 && t[t.length - 1])) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2]) _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                } catch (e) {
                    op = [6, e];
                    y = 0;
                } finally {
                    f = t = 0;
                }
            if (op[0] & 5) throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    function __exportStar(m, exports) {
        for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
    }
    function __values(o) {
        var s = typeof Symbol === 'function' && Symbol.iterator,
            m = s && o[s],
            i = 0;
        if (m) return m.call(o);
        if (o && typeof o.length === 'number')
            return {
                next: function () {
                    if (o && i >= o.length) o = void 0;
                    return { value: o && o[i++], done: !o };
                },
            };
        throw new TypeError(s ? 'Object is not iterable.' : 'Symbol.iterator is not defined.');
    }
    function __read(o, n) {
        var m = typeof Symbol === 'function' && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o),
            r,
            ar = [],
            e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        } catch (error) {
            e = { error: error };
        } finally {
            try {
                if (r && !r.done && (m = i['return'])) m.call(i);
            } finally {
                if (e) throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++) for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++) r[k] = a[j];
        return r;
    }
    function __await(v) {
        return this instanceof __await ? ((this.v = v), this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator) throw new TypeError('Symbol.asyncIterator is not defined.');
        var g = generator.apply(thisArg, _arguments || []),
            i,
            q = [];
        return (
            (i = {}),
            verb('next'),
            verb('throw'),
            verb('return'),
            (i[Symbol.asyncIterator] = function () {
                return this;
            }),
            i
        );
        function verb(n) {
            if (g[n])
                i[n] = function (v) {
                    return new Promise(function (a, b) {
                        q.push([n, v, a, b]) > 1 || resume(n, v);
                    });
                };
        }
        function resume(n, v) {
            try {
                step(g[n](v));
            } catch (e) {
                settle(q[0][3], e);
            }
        }
        function step(r) {
            r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r);
        }
        function fulfill(value) {
            resume('next', value);
        }
        function reject(value) {
            resume('throw', value);
        }
        function settle(f, v) {
            if ((f(v), q.shift(), q.length)) resume(q[0][0], q[0][1]);
        }
    }
    function __asyncDelegator(o) {
        var i, p;
        return (
            (i = {}),
            verb('next'),
            verb('throw', function (e) {
                throw e;
            }),
            verb('return'),
            (i[Symbol.iterator] = function () {
                return this;
            }),
            i
        );
        function verb(n, f) {
            i[n] = o[n]
                ? function (v) {
                      return (p = !p) ? { value: __await(o[n](v)), done: n === 'return' } : f ? f(v) : v;
                  }
                : f;
        }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator) throw new TypeError('Symbol.asyncIterator is not defined.');
        var m = o[Symbol.asyncIterator],
            i;
        return m
            ? m.call(o)
            : ((o = typeof __values === 'function' ? __values(o) : o[Symbol.iterator]()),
              (i = {}),
              verb('next'),
              verb('throw'),
              verb('return'),
              (i[Symbol.asyncIterator] = function () {
                  return this;
              }),
              i);
        function verb(n) {
            i[n] =
                o[n] &&
                function (v) {
                    return new Promise(function (resolve, reject) {
                        (v = o[n](v)), settle(resolve, reject, v.done, v.value);
                    });
                };
        }
        function settle(resolve, reject, d, v) {
            Promise.resolve(v).then(function (v) {
                resolve({ value: v, done: d });
            }, reject);
        }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, 'raw', { value: raw });
        } else {
            cooked.raw = raw;
        }
        return cooked;
    }
    function __importStar(mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
        result.default = mod;
        return result;
    }
    function __importDefault(mod) {
        return mod && mod.__esModule ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError('attempted to get private field on non-instance');
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError('attempted to set private field on non-instance');
        }
        privateMap.set(receiver, value);
        return value;
    }

    var OwlLogger = /** @class */ (function () {
        function OwlLogger(errorHandler) {
            this.errorHandler = errorHandler;
        }
        OwlLogger.prototype.log = function (value) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            if (core.isDevMode()) {
                console.log.apply(console, __spread([value], rest));
            }
        };
        OwlLogger.prototype.error = function (error) {
            this.errorHandler.handleError(error);
        };
        OwlLogger.prototype.warn = function (value) {
            var rest = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                rest[_i - 1] = arguments[_i];
            }
            console.warn.apply(console, __spread([value], rest));
        };
        return OwlLogger;
    })();
    OwlLogger.decorators = [{ type: core.Injectable }];
    OwlLogger.ctorParameters = function () {
        return [{ type: core.ErrorHandler }];
    };

    /**
     * Current state information and their tags.
     */
    var States = /** @class */ (function () {
        function States() {}
        return States;
    })();
    /**
     * Enumeration for types.
     * @enum {String}
     */
    var Type;
    (function (Type) {
        Type['Event'] = 'event';
        Type['State'] = 'state';
    })(Type || (Type = {}));
    /**
     * Enumeration for width.
     * @enum {String}
     */
    var Width;
    (function (Width) {
        Width['Default'] = 'default';
        Width['Inner'] = 'inner';
        Width['Outer'] = 'outer';
    })(Width || (Width = {}));
    /**
     * Model for coords of .owl-stage
     */
    var Coords = /** @class */ (function () {
        function Coords() {}
        return Coords;
    })();
    /**
     * Model for all current data of carousel
     */
    var CarouselCurrentData = /** @class */ (function () {
        function CarouselCurrentData() {}
        return CarouselCurrentData;
    })();
    var CarouselService = /** @class */ (function () {
        function CarouselService(logger) {
            var _this = this;
            this.logger = logger;
            /**
             * Subject for passing data needed for managing View
             */
            this._viewSettingsShipper$ = new rxjs.Subject();
            /**
             * Subject for notification when the carousel got initializes
             */
            this._initializedCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the carousel's settings start changinf
             */
            this._changeSettingsCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the carousel's settings have changed
             */
            this._changedSettingsCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the carousel starts translating or moving
             */
            this._translateCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the carousel stopped translating or moving
             */
            this._translatedCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the carousel's rebuilding caused by 'resize' event starts
             */
            this._resizeCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification  when the carousel's rebuilding caused by 'resize' event is ended
             */
            this._resizedCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the refresh of carousel starts
             */
            this._refreshCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the refresh of carousel is ended
             */
            this._refreshedCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the dragging of carousel starts
             */
            this._dragCarousel$ = new rxjs.Subject();
            /**
             * Subject for notification when the dragging of carousel is ended
             */
            this._draggedCarousel$ = new rxjs.Subject();
            /**
             * Current settings for the carousel.
             */
            this.settings = {
                items: 0,
            };
            /**
             * Initial data for setting classes to element .owl-carousel
             */
            this.owlDOMData = {
                rtl: false,
                isResponsive: false,
                isRefreshed: false,
                isLoaded: false,
                isLoading: false,
                isMouseDragable: false,
                isGrab: false,
                isTouchDragable: false,
            };
            /**
             * Initial data of .owl-stage
             */
            this.stageData = {
                transform: 'translate3d(0px,0px,0px)',
                transition: '0s',
                width: 0,
                paddingL: 0,
                paddingR: 0,
            };
            /**
             * All real items.
             */
            this._items = []; // is equal to this.slides
            /**
             * Array with width of every slide.
             */
            this._widths = [];
            /**
             * Currently suppressed events to prevent them from beeing retriggered.
             */
            this._supress = {};
            /**
             * References to the running plugins of this carousel.
             */
            this._plugins = {};
            /**
             * Absolute current position.
             */
            this._current = null;
            /**
             * All cloned items.
             */
            this._clones = [];
            /**
             * Merge values of all items.
             * @todo Maybe this could be part of a plugin.
             */
            this._mergers = [];
            /**
             * Animation speed in milliseconds.
             */
            this._speed = null;
            /**
             * Coordinates of all items in pixel.
             * @todo The name of this member is missleading.
             */
            this._coordinates = [];
            /**
             * Current breakpoint.
             * @todo Real media queries would be nice.
             */
            this._breakpoint = null;
            /**
             * Prefix for id of cloned slides
             */
            this.clonedIdPrefix = 'cloned-';
            /**
             * Current options set by the caller including defaults.
             */
            this._options = {};
            /**
             * Invalidated parts within the update process.
             */
            this._invalidated = {};
            /**
             * Current state information and their tags.
             */
            this._states = {
                current: {},
                tags: {
                    initializing: ['busy'],
                    animating: ['busy'],
                    dragging: ['interacting'],
                },
            };
            /**
             * Ordered list of workers for the update process.
             */
            this._pipe = [
                // {
                //   filter: ['width', 'settings'],
                //   run: () => {
                //     this._width = this.carouselWindowWidth;
                //   }
                // },
                {
                    filter: ['width', 'items', 'settings'],
                    run: function (cache) {
                        cache.current = _this._items && _this._items[_this.relative(_this._current)].id;
                    },
                },
                // {
                //   filter: ['items', 'settings'],
                //   run: function() {
                //     // this.$stage.children('.cloned').remove();
                //   }
                // },
                {
                    filter: ['width', 'items', 'settings'],
                    run: function (cache) {
                        var margin = _this.settings.margin || '',
                            grid = !_this.settings.autoWidth,
                            rtl = _this.settings.rtl,
                            css = {
                                'margin-left': rtl ? margin : '',
                                'margin-right': rtl ? '' : margin,
                            };
                        if (!grid) {
                            _this.slidesData.forEach(function (slide) {
                                slide.marginL = css['margin-left'];
                                slide.marginR = css['margin-right'];
                            });
                        }
                        cache.css = css;
                    },
                },
                {
                    filter: ['width', 'items', 'settings'],
                    run: function (cache) {
                        var width = +(_this.width() / _this.settings.items).toFixed(3) - _this.settings.margin,
                            grid = !_this.settings.autoWidth,
                            widths = [];
                        var merge = null,
                            iterator = _this._items.length;
                        cache.items = {
                            merge: false,
                            width: width,
                        };
                        while (iterator--) {
                            merge = _this._mergers[iterator];
                            merge = (_this.settings.mergeFit && Math.min(merge, _this.settings.items)) || merge;
                            cache.items.merge = merge > 1 || cache.items.merge;
                            widths[iterator] = !grid ? (_this._items[iterator].width ? _this._items[iterator].width : width) : width * merge;
                        }
                        _this._widths = widths;
                        _this.slidesData.forEach(function (slide, i) {
                            slide.width = _this._widths[i];
                            slide.marginR = cache.css['margin-right'];
                            slide.marginL = cache.css['margin-left'];
                        });
                    },
                },
                {
                    filter: ['items', 'settings'],
                    run: function () {
                        var clones = [],
                            items = _this._items,
                            settings = _this.settings,
                            // TODO: Should be computed from number of min width items in stage
                            view = Math.max(settings.items * 2, 4),
                            size = Math.ceil(items.length / 2) * 2;
                        var append = [],
                            prepend = [],
                            repeat = settings.loop && items.length ? (settings.rewind ? view : Math.max(view, size)) : 0;
                        repeat /= 2;
                        while (repeat--) {
                            // Switch to only using appended clones
                            clones.push(_this.normalize(clones.length / 2, true));
                            append.push(Object.assign({}, _this.slidesData[clones[clones.length - 1]]));
                            clones.push(_this.normalize(items.length - 1 - (clones.length - 1) / 2, true));
                            prepend.unshift(Object.assign({}, _this.slidesData[clones[clones.length - 1]]));
                        }
                        _this._clones = clones;
                        append = append.map(function (slide) {
                            slide.id = '' + _this.clonedIdPrefix + slide.id;
                            slide.isActive = false;
                            slide.isCloned = true;
                            return slide;
                        });
                        prepend = prepend.map(function (slide) {
                            slide.id = '' + _this.clonedIdPrefix + slide.id;
                            slide.isActive = false;
                            slide.isCloned = true;
                            return slide;
                        });
                        _this.slidesData = prepend.concat(_this.slidesData).concat(append);
                    },
                },
                {
                    filter: ['width', 'items', 'settings'],
                    run: function () {
                        var rtl = _this.settings.rtl ? 1 : -1,
                            size = _this._clones.length + _this._items.length,
                            coordinates = [];
                        var iterator = -1,
                            previous = 0,
                            current = 0;
                        while (++iterator < size) {
                            previous = coordinates[iterator - 1] || 0;
                            current = _this._widths[_this.relative(iterator)] + _this.settings.margin;
                            coordinates.push(previous + current * rtl);
                        }
                        _this._coordinates = coordinates;
                    },
                },
                {
                    filter: ['width', 'items', 'settings'],
                    run: function () {
                        var padding = _this.settings.stagePadding,
                            coordinates = _this._coordinates,
                            css = {
                                width: Math.ceil(Math.abs(coordinates[coordinates.length - 1])) + padding * 2,
                                'padding-left': padding || '',
                                'padding-right': padding || '',
                            };
                        _this.stageData.width = css.width; // use this property in *ngIf directive for .owl-stage element
                        _this.stageData.paddingL = css['padding-left'];
                        _this.stageData.paddingR = css['padding-right'];
                    },
                },
                {
                    //   filter: [ 'width', 'items', 'settings' ],
                    //   run: cache => {
                    // 		// this method sets the width for every slide, but I set it in different way earlier
                    // 		const grid = !this.settings.autoWidth,
                    // 		items = this.$stage.children(); // use this.slidesData
                    //     let iterator = this._coordinates.length;
                    //     if (grid && cache.items.merge) {
                    //       while (iterator--) {
                    //         cache.css.width = this._widths[this.relative(iterator)];
                    //         items.eq(iterator).css(cache.css);
                    //       }
                    //     } else if (grid) {
                    //       cache.css.width = cache.items.width;
                    //       items.css(cache.css);
                    //     }
                    //   }
                    // }, {
                    //   filter: [ 'items' ],
                    //   run: function() {
                    //     this._coordinates.length < 1 && this.$stage.removeAttr('style');
                    //   }
                    // }, {
                    filter: ['width', 'items', 'settings'],
                    run: function (cache) {
                        var current = cache.current
                            ? _this.slidesData.findIndex(function (slide) {
                                  return slide.id === cache.current;
                              })
                            : 0;
                        current = Math.max(_this.minimum(), Math.min(_this.maximum(), current));
                        _this.reset(current);
                    },
                },
                {
                    filter: ['position'],
                    run: function () {
                        _this.animate(_this.coordinates(_this._current));
                    },
                },
                {
                    filter: ['width', 'position', 'items', 'settings'],
                    run: function () {
                        var rtl = _this.settings.rtl ? 1 : -1,
                            padding = _this.settings.stagePadding * 2,
                            matches = [];
                        var begin, end, inner, outer, i, n;
                        begin = _this.coordinates(_this.current());
                        if (typeof begin === 'number') {
                            begin += padding;
                        } else {
                            begin = 0;
                        }
                        end = begin + _this.width() * rtl;
                        if (rtl === -1 && _this.settings.center) {
                            var result = _this._coordinates.filter(function (element) {
                                return _this.settings.items % 2 === 1 ? element >= begin : element > begin;
                            });
                            begin = result.length ? result[result.length - 1] : begin;
                        }
                        for (i = 0, n = _this._coordinates.length; i < n; i++) {
                            inner = Math.ceil(_this._coordinates[i - 1] || 0);
                            outer = Math.ceil(Math.abs(_this._coordinates[i]) + padding * rtl);
                            if ((_this._op(inner, '<=', begin) && _this._op(inner, '>', end)) || (_this._op(outer, '<', begin) && _this._op(outer, '>', end))) {
                                matches.push(i);
                            }
                        }
                        _this.slidesData.forEach(function (slide) {
                            slide.isActive = false;
                            return slide;
                        });
                        matches.forEach(function (item) {
                            _this.slidesData[item].isActive = true;
                        });
                        if (_this.settings.center) {
                            _this.slidesData.forEach(function (slide) {
                                slide.isCentered = false;
                                return slide;
                            });
                            _this.slidesData[_this.current()].isCentered = true;
                        }
                    },
                },
            ];
        }
        Object.defineProperty(CarouselService.prototype, 'invalidated', {
            // Is needed for tests
            get: function () {
                return this._invalidated;
            },
            enumerable: false,
            configurable: true,
        });
        Object.defineProperty(CarouselService.prototype, 'states', {
            // is needed for tests
            get: function () {
                return this._states;
            },
            enumerable: false,
            configurable: true,
        });
        /**
         * Makes _viewSettingsShipper$ Subject become Observable
         * @returns Observable of _viewSettingsShipper$ Subject
         */
        CarouselService.prototype.getViewCurSettings = function () {
            return this._viewSettingsShipper$.asObservable();
        };
        /**
         * Makes _initializedCarousel$ Subject become Observable
         * @returns Observable of _initializedCarousel$ Subject
         */
        CarouselService.prototype.getInitializedState = function () {
            return this._initializedCarousel$.asObservable();
        };
        /**
         * Makes _changeSettingsCarousel$ Subject become Observable
         * @returns Observable of _changeSettingsCarousel$ Subject
         */
        CarouselService.prototype.getChangeState = function () {
            return this._changeSettingsCarousel$.asObservable();
        };
        /**
         * Makes _changedSettingsCarousel$ Subject become Observable
         * @returns Observable of _changedSettingsCarousel$ Subject
         */
        CarouselService.prototype.getChangedState = function () {
            return this._changedSettingsCarousel$.asObservable();
        };
        /**
         * Makes _translateCarousel$ Subject become Observable
         * @returns Observable of _translateCarousel$ Subject
         */
        CarouselService.prototype.getTranslateState = function () {
            return this._translateCarousel$.asObservable();
        };
        /**
         * Makes _translatedCarousel$ Subject become Observable
         * @returns Observable of _translatedCarousel$ Subject
         */
        CarouselService.prototype.getTranslatedState = function () {
            return this._translatedCarousel$.asObservable();
        };
        /**
         * Makes _resizeCarousel$ Subject become Observable
         * @returns Observable of _resizeCarousel$ Subject
         */
        CarouselService.prototype.getResizeState = function () {
            return this._resizeCarousel$.asObservable();
        };
        /**
         * Makes _resizedCarousel$ Subject become Observable
         * @returns Observable of _resizedCarousel$ Subject
         */
        CarouselService.prototype.getResizedState = function () {
            return this._resizedCarousel$.asObservable();
        };
        /**
         * Makes _refreshCarousel$ Subject become Observable
         * @returns Observable of _refreshCarousel$ Subject
         */
        CarouselService.prototype.getRefreshState = function () {
            return this._refreshCarousel$.asObservable();
        };
        /**
         * Makes _refreshedCarousel$ Subject become Observable
         * @returns Observable of _refreshedCarousel$ Subject
         */
        CarouselService.prototype.getRefreshedState = function () {
            return this._refreshedCarousel$.asObservable();
        };
        /**
         * Makes _dragCarousel$ Subject become Observable
         * @returns Observable of _dragCarousel$ Subject
         */
        CarouselService.prototype.getDragState = function () {
            return this._dragCarousel$.asObservable();
        };
        /**
         * Makes _draggedCarousel$ Subject become Observable
         * @returns Observable of _draggedCarousel$ Subject
         */
        CarouselService.prototype.getDraggedState = function () {
            return this._draggedCarousel$.asObservable();
        };
        /**
         * Setups custom options expanding default options
         * @param options custom options
         */
        CarouselService.prototype.setOptions = function (options) {
            var configOptions = new OwlCarouselOConfig();
            var checkedOptions = this._validateOptions(options, configOptions);
            this._options = Object.assign(Object.assign({}, configOptions), checkedOptions);
        };
        /**
         * Checks whether user's option are set properly. Cheking is based on typings;
         * @param options options set by user
         * @param configOptions default options
         * @returns checked and modified (if it's needed) user's options
         *
         * Notes:
         * 	- if user set option with wrong type, it'll be written in console
         */
        CarouselService.prototype._validateOptions = function (options, configOptions) {
            var _this = this;
            var checkedOptions = Object.assign({}, options);
            var mockedTypes = new OwlOptionsMockedTypes();
            var setRightOption = function (type, key) {
                _this.logger.log(
                    'options.' + key + ' must be type of ' + type + '; ' + key + '=' + options[key] + ' skipped to defaults: ' + key + '=' + configOptions[key],
                );
                return configOptions[key];
            };
            var _loop_1 = function (key) {
                if (checkedOptions.hasOwnProperty(key)) {
                    // condition could be shortened but it gets harder for understanding
                    if (mockedTypes[key] === 'number') {
                        if (this_1._isNumeric(checkedOptions[key])) {
                            checkedOptions[key] = +checkedOptions[key];
                            checkedOptions[key] =
                                key === 'items' ? this_1._validateItems(checkedOptions[key], checkedOptions.skip_validateItems) : checkedOptions[key];
                        } else {
                            checkedOptions[key] = setRightOption(mockedTypes[key], key);
                        }
                    } else if (mockedTypes[key] === 'boolean' && typeof checkedOptions[key] !== 'boolean') {
                        checkedOptions[key] = setRightOption(mockedTypes[key], key);
                    } else if (mockedTypes[key] === 'number|boolean' && !this_1._isNumberOrBoolean(checkedOptions[key])) {
                        checkedOptions[key] = setRightOption(mockedTypes[key], key);
                    } else if (mockedTypes[key] === 'number|string' && !this_1._isNumberOrString(checkedOptions[key])) {
                        checkedOptions[key] = setRightOption(mockedTypes[key], key);
                    } else if (mockedTypes[key] === 'string|boolean' && !this_1._isStringOrBoolean(checkedOptions[key])) {
                        checkedOptions[key] = setRightOption(mockedTypes[key], key);
                    } else if (mockedTypes[key] === 'string[]') {
                        if (Array.isArray(checkedOptions[key])) {
                            var isString_1 = false;
                            checkedOptions[key].forEach(function (element) {
                                isString_1 = typeof element === 'string' ? true : false;
                            });
                            if (!isString_1) {
                                checkedOptions[key] = setRightOption(mockedTypes[key], key);
                            }
                        } else {
                            checkedOptions[key] = setRightOption(mockedTypes[key], key);
                        }
                    }
                }
            };
            var this_1 = this;
            for (var key in checkedOptions) {
                _loop_1(key);
            }
            return checkedOptions;
        };
        /**
         * Checks the option `items` set by user and if it bigger than number of slides, the function returns number of slides
         * @param items option items set by user
         * @param skip_validateItems option `skip_validateItems` set by user
         * @returns right number of items
         */
        CarouselService.prototype._validateItems = function (items, skip_validateItems) {
            var result = items;
            if (items > this._items.length) {
                if (skip_validateItems) {
                    this.logger.log("The option 'items' in your options is bigger than the number of slides. The navigation got disabled");
                } else {
                    result = this._items.length;
                    this.logger.log(
                        "The option 'items' in your options is bigger than the number of slides. This option is updated to the current number of slides and the navigation got disabled",
                    );
                }
            } else {
                if (items === this._items.length && (this.settings.dots || this.settings.nav)) {
                    this.logger.log("Option 'items' in your options is equal to the number of slides. So the navigation got disabled");
                }
            }
            return result;
        };
        /**
         * Set current width of carousel
         * @param width width of carousel Window
         */
        CarouselService.prototype.setCarouselWidth = function (width) {
            this._width = width;
        };
        /**
         * Setups the current settings.
         * @todo Remove responsive classes. Why should adaptive designs be brought into IE8?
         * @todo Support for media queries by using `matchMedia` would be nice.
         * @param carouselWidth width of carousel
         * @param slides array of slides
         * @param options options set by user
         */
        CarouselService.prototype.setup = function (carouselWidth, slides, options) {
            this.setCarouselWidth(carouselWidth);
            this.setItems(slides);
            this._defineSlidesData();
            this.setOptions(options);
            this.settings = Object.assign({}, this._options);
            this.setOptionsForViewport();
            this._trigger('change', { property: { name: 'settings', value: this.settings } });
            this.invalidate('settings'); // must be call of this function;
            this._trigger('changed', { property: { name: 'settings', value: this.settings } });
        };
        /**
         * Set options for current viewport
         */
        CarouselService.prototype.setOptionsForViewport = function () {
            var _this = this;
            var viewport = this._width,
                overwrites = this._options.responsive;
            var match = -1;
            if (!Object.keys(overwrites).length) {
                return;
            }
            if (!viewport) {
                this.settings.items = 1;
                return;
            }
            for (var key in overwrites) {
                if (overwrites.hasOwnProperty(key)) {
                    if (+key <= viewport && +key > match) {
                        match = Number(key);
                    }
                }
            }
            this.settings = Object.assign(Object.assign(Object.assign({}, this._options), overwrites[match]), {
                items:
                    overwrites[match] && overwrites[match].items
                        ? this._validateItems(overwrites[match].items, this._options.skip_validateItems)
                        : this._options.items,
            });
            // if (typeof this.settings.stagePadding === 'function') {
            // 	this.settings.stagePadding = this.settings.stagePadding();
            // }
            delete this.settings.responsive;
            this.owlDOMData.isResponsive = true;
            this.owlDOMData.isMouseDragable = this.settings.mouseDrag;
            this.owlDOMData.isTouchDragable = this.settings.touchDrag;
            var mergers = [];
            this._items.forEach(function (item) {
                var mergeN = _this.settings.merge ? item.dataMerge : 1;
                mergers.push(mergeN);
            });
            this._mergers = mergers;
            this._breakpoint = match;
            this.invalidate('settings');
        };
        /**
         * Initializes the carousel.
         * @param slides array of CarouselSlideDirective
         */
        CarouselService.prototype.initialize = function (slides) {
            var _this = this;
            this.enter('initializing');
            // this.trigger('initialize');
            this.owlDOMData.rtl = this.settings.rtl;
            if (this._mergers.length) {
                this._mergers = [];
            }
            slides.forEach(function (item) {
                var mergeN = _this.settings.merge ? item.dataMerge : 1;
                _this._mergers.push(mergeN);
            });
            this._clones = [];
            this.reset(this._isNumeric(this.settings.startPosition) ? +this.settings.startPosition : 0);
            this.invalidate('items');
            this.refresh();
            this.owlDOMData.isLoaded = true;
            this.owlDOMData.isMouseDragable = this.settings.mouseDrag;
            this.owlDOMData.isTouchDragable = this.settings.touchDrag;
            this.sendChanges();
            this.leave('initializing');
            this._trigger('initialized');
        };
        /**
         * Sends all data needed for View
         */
        CarouselService.prototype.sendChanges = function () {
            this._viewSettingsShipper$.next({
                owlDOMData: this.owlDOMData,
                stageData: this.stageData,
                slidesData: this.slidesData,
                navData: this.navData,
                dotsData: this.dotsData,
            });
        };
        /**
         * Updates option logic if necessery
         */
        CarouselService.prototype._optionsLogic = function () {
            if (this.settings.autoWidth) {
                this.settings.stagePadding = 0;
                this.settings.merge = false;
            }
        };
        /**
         * Updates the view
         */
        CarouselService.prototype.update = function () {
            var _this = this;
            var i = 0;
            var n = this._pipe.length,
                filter = function (item) {
                    return _this._invalidated[item];
                },
                cache = {};
            while (i < n) {
                var filteredPipe = this._pipe[i].filter.filter(filter);
                if (this._invalidated.all || filteredPipe.length > 0) {
                    this._pipe[i].run(cache);
                }
                i++;
            }
            this.slidesData.forEach(function (slide) {
                return (slide.classes = _this.setCurSlideClasses(slide));
            });
            this.sendChanges();
            this._invalidated = {};
            if (!this.is('valid')) {
                this.enter('valid');
            }
        };
        /**
         * Gets the width of the view.
         * @param [dimension=Width.Default] The dimension to return
         * @returns The width of the view in pixel.
         */
        CarouselService.prototype.width = function (dimension) {
            dimension = dimension || Width.Default;
            switch (dimension) {
                case Width.Inner:
                case Width.Outer:
                    return this._width;
                default:
                    return this._width - this.settings.stagePadding * 2 + this.settings.margin;
            }
        };
        /**
         * Refreshes the carousel primarily for adaptive purposes.
         */
        CarouselService.prototype.refresh = function () {
            this.enter('refreshing');
            this._trigger('refresh');
            this._defineSlidesData();
            this.setOptionsForViewport();
            this._optionsLogic();
            // this.$element.addClass(this.options.refreshClass);
            this.update();
            // this.$element.removeClass(this.options.refreshClass);
            this.leave('refreshing');
            this._trigger('refreshed');
        };
        /**
         * Checks window `resize` event.
         * @param curWidth width of .owl-carousel
         */
        CarouselService.prototype.onResize = function (curWidth) {
            if (!this._items.length) {
                return false;
            }
            this.setCarouselWidth(curWidth);
            this.enter('resizing');
            // if (this.trigger('resize').isDefaultPrevented()) {
            // 	this.leave('resizing');
            // 	return false;
            // }
            this._trigger('resize');
            this.invalidate('width');
            this.refresh();
            this.leave('resizing');
            this._trigger('resized');
        };
        /**
         * Prepares data for dragging carousel. It starts after firing `touchstart` and `mousedown` events.
         * @todo Horizontal swipe threshold as option
         * @todo #261
         * @param event - The event arguments.
         * @returns stage - object with 'x' and 'y' coordinates of .owl-stage
         */
        CarouselService.prototype.prepareDragging = function (event) {
            var stage = null,
                transformArr;
            // could be 5 commented lines below; However there's stage transform in stageData and in updates after each move of stage
            // stage = getComputedStyle(this.el.nativeElement).transform.replace(/.*\(|\)| /g, '').split(',');
            // stage = {
            //   x: stage[stage.length === 16 ? 12 : 4],
            //   y: stage[stage.length === 16 ? 13 : 5]
            // };
            transformArr = this.stageData.transform.replace(/.*\(|\)| |[^,-\d]\w|\)/g, '').split(',');
            stage = {
                x: +transformArr[0],
                y: +transformArr[1],
            };
            if (this.is('animating')) {
                this.invalidate('position');
            }
            if (event.type === 'mousedown') {
                this.owlDOMData.isGrab = true;
            }
            this.speed(0);
            return stage;
        };
        /**
         * Enters into a 'dragging' state
         */
        CarouselService.prototype.enterDragging = function () {
            this.enter('dragging');
            this._trigger('drag');
        };
        /**
         * Defines new coords for .owl-stage while dragging it
         * @todo #261
         * @param event the event arguments.
         * @param dragData initial data got after starting dragging
         * @returns coords or false
         */
        CarouselService.prototype.defineNewCoordsDrag = function (event, dragData) {
            var minimum = null,
                maximum = null,
                pull = null;
            var delta = this.difference(dragData.pointer, this.pointer(event)),
                stage = this.difference(dragData.stage.start, delta);
            if (!this.is('dragging')) {
                return false;
            }
            if (this.settings.loop) {
                minimum = this.coordinates(this.minimum());
                maximum = +this.coordinates(this.maximum() + 1) - minimum;
                stage.x = ((((stage.x - minimum) % maximum) + maximum) % maximum) + minimum;
            } else {
                minimum = this.settings.rtl ? this.coordinates(this.maximum()) : this.coordinates(this.minimum());
                maximum = this.settings.rtl ? this.coordinates(this.minimum()) : this.coordinates(this.maximum());
                pull = this.settings.pullDrag ? (-1 * delta.x) / 5 : 0;
                stage.x = Math.max(Math.min(stage.x, minimum + pull), maximum + pull);
            }
            return stage;
        };
        /**
         * Finishes dragging of carousel when `touchend` and `mouseup` events fire.
         * @todo #261
         * @todo Threshold for click event
         * @param event the event arguments.
         * @param dragObj the object with dragging settings and states
         * @param clickAttacher function which attaches click handler to slide or its children elements in order to prevent event bubling
         */
        CarouselService.prototype.finishDragging = function (event, dragObj, clickAttacher) {
            var directions = ['right', 'left'],
                delta = this.difference(dragObj.pointer, this.pointer(event)),
                stage = dragObj.stage.current,
                direction = directions[+(this.settings.rtl ? delta.x < +this.settings.rtl : delta.x > +this.settings.rtl)];
            var currentSlideI, current, newCurrent;
            if ((delta.x !== 0 && this.is('dragging')) || !this.is('valid')) {
                this.speed(+this.settings.dragEndSpeed || this.settings.smartSpeed);
                currentSlideI = this.closest(stage.x, delta.x !== 0 ? direction : dragObj.direction);
                current = this.current();
                newCurrent = this.current(currentSlideI === -1 ? undefined : currentSlideI);
                if (current !== newCurrent) {
                    this.invalidate('position');
                    this.update();
                }
                dragObj.direction = direction;
                if (Math.abs(delta.x) > 3 || new Date().getTime() - dragObj.time > 300) {
                    clickAttacher();
                }
            }
            if (!this.is('dragging')) {
                return;
            }
            this.leave('dragging');
            this._trigger('dragged');
        };
        /**
         * Gets absolute position of the closest item for a coordinate.
         * @todo Setting `freeDrag` makes `closest` not reusable. See #165.
         * @param coordinate The coordinate in pixel.
         * @param direction The direction to check for the closest item. Ether `left` or `right`.
         * @returns The absolute position of the closest item.
         */
        CarouselService.prototype.closest = function (coordinate, direction) {
            var pull = 30,
                width = this.width();
            var coordinates = this.coordinates(),
                position = -1;
            if (this.settings.center) {
                coordinates = coordinates.map(function (item) {
                    if (item === 0) {
                        item += 0.000001;
                    }
                    return item;
                });
            }
            // option 'freeDrag' doesn't have realization and using it here creates problem:
            // variable 'position' stays unchanged (it equals -1 at the begging) and thus method returns -1
            // Returning value is consumed by method current(), which taking -1 as argument calculates the index of new current slide
            // In case of having 5 slides ans 'loop=false; calling 'current(-1)' sets props '_current' as 4. Just last slide remains visible instead of 3 last slides.
            // if (!this.settings.freeDrag) {
            // check closest item
            for (var i = 0; i < coordinates.length; i++) {
                if (direction === 'left' && coordinate > coordinates[i] - pull && coordinate < coordinates[i] + pull) {
                    position = i;
                    // on a right pull, check on previous index
                    // to do so, subtract width from value and set position = index + 1
                } else if (direction === 'right' && coordinate > coordinates[i] - width - pull && coordinate < coordinates[i] - width + pull) {
                    position = i + 1;
                } else if (this._op(coordinate, '<', coordinates[i]) && this._op(coordinate, '>', coordinates[i + 1] || coordinates[i] - width)) {
                    position = direction === 'left' ? i + 1 : i;
                } else if (direction === null && coordinate > coordinates[i] - pull && coordinate < coordinates[i] + pull) {
                    position = i;
                }
                if (position !== -1) {
                    break;
                }
            }
            // }
            if (!this.settings.loop) {
                // non loop boundries
                if (this._op(coordinate, '>', coordinates[this.minimum()])) {
                    position = coordinate = this.minimum();
                } else if (this._op(coordinate, '<', coordinates[this.maximum()])) {
                    position = coordinate = this.maximum();
                }
            }
            return position;
        };
        /**
         * Animates the stage.
         * @todo #270
         * @param coordinate The coordinate in pixels.
         */
        CarouselService.prototype.animate = function (coordinate) {
            var animate = this.speed() > 0;
            if (this.is('animating')) {
                this.onTransitionEnd();
            }
            if (animate) {
                this.enter('animating');
                this._trigger('translate');
            }
            this.stageData.transform = 'translate3d(' + coordinate + 'px,0px,0px)';
            this.stageData.transition = this.speed() / 1000 + 's' + (this.settings.slideTransition ? ' ' + this.settings.slideTransition : '');
            // also there was transition by means of JQuery.animate or css-changing property left
        };
        /**
         * Checks whether the carousel is in a specific state or not.
         * @param state The state to check.
         * @returns The flag which indicates if the carousel is busy.
         */
        CarouselService.prototype.is = function (state) {
            return this._states.current[state] && this._states.current[state] > 0;
        };
        /**
         * Sets the absolute position of the current item.
         * @param position The new absolute position or nothing to leave it unchanged.
         * @returns The absolute position of the current item.
         */
        CarouselService.prototype.current = function (position) {
            if (position === undefined) {
                return this._current;
            }
            if (this._items.length === 0) {
                return undefined;
            }
            position = this.normalize(position);
            if (this._current !== position) {
                var event = this._trigger('change', { property: { name: 'position', value: position } });
                // if (event.data !== undefined) {
                // 	position = this.normalize(event.data);
                // }
                this._current = position;
                this.invalidate('position');
                this._trigger('changed', { property: { name: 'position', value: this._current } });
            }
            return this._current;
        };
        /**
         * Invalidates the given part of the update routine.
         * @param part The part to invalidate.
         * @returns The invalidated parts.
         */
        CarouselService.prototype.invalidate = function (part) {
            if (typeof part === 'string') {
                this._invalidated[part] = true;
                if (this.is('valid')) {
                    this.leave('valid');
                }
            }
            return Object.keys(this._invalidated);
        };
        /**
         * Resets the absolute position of the current item.
         * @param position the absolute position of the new item.
         */
        CarouselService.prototype.reset = function (position) {
            position = this.normalize(position);
            if (position === undefined) {
                return;
            }
            this._speed = 0;
            this._current = position;
            this._suppress(['translate', 'translated']);
            this.animate(this.coordinates(position));
            this._release(['translate', 'translated']);
        };
        /**
         * Normalizes an absolute or a relative position of an item.
         * @param position The absolute or relative position to normalize.
         * @param relative Whether the given position is relative or not.
         * @returns The normalized position.
         */
        CarouselService.prototype.normalize = function (position, relative) {
            var n = this._items.length,
                m = relative ? 0 : this._clones.length;
            if (!this._isNumeric(position) || n < 1) {
                position = undefined;
            } else if (position < 0 || position >= n + m) {
                position = ((((position - m / 2) % n) + n) % n) + m / 2;
            }
            return position;
        };
        /**
         * Converts an absolute position of an item into a relative one.
         * @param position The absolute position to convert.
         * @returns The converted position.
         */
        CarouselService.prototype.relative = function (position) {
            position -= this._clones.length / 2;
            return this.normalize(position, true);
        };
        /**
         * Gets the maximum position for the current item.
         * @param relative Whether to return an absolute position or a relative position.
         * @returns number of maximum position
         */
        CarouselService.prototype.maximum = function (relative) {
            if (relative === void 0) {
                relative = false;
            }
            var settings = this.settings;
            var maximum = this._coordinates.length,
                iterator,
                reciprocalItemsWidth,
                elementWidth;
            if (settings.loop) {
                maximum = this._clones.length / 2 + this._items.length - 1;
            } else if (settings.autoWidth || settings.merge) {
                iterator = this._items.length;
                reciprocalItemsWidth = this.slidesData[--iterator].width;
                elementWidth = this._width;
                while (iterator--) {
                    // it could be use this._items instead of this.slidesData;
                    reciprocalItemsWidth += +this.slidesData[iterator].width + this.settings.margin;
                    if (reciprocalItemsWidth > elementWidth) {
                        break;
                    }
                }
                maximum = iterator + 1;
            } else if (settings.center) {
                maximum = this._items.length - 1;
            } else {
                maximum = this._items.length - settings.items;
            }
            if (relative) {
                maximum -= this._clones.length / 2;
            }
            return Math.max(maximum, 0);
        };
        /**
         * Gets the minimum position for the current item.
         * @param relative Whether to return an absolute position or a relative position.
         * @returns number of minimum position
         */
        CarouselService.prototype.minimum = function (relative) {
            if (relative === void 0) {
                relative = false;
            }
            return relative ? 0 : this._clones.length / 2;
        };
        /**
         * Gets an item at the specified relative position.
         * @param position The relative position of the item.
         * @returns The item at the given position or all items if no position was given.
         */
        CarouselService.prototype.items = function (position) {
            if (position === undefined) {
                return this._items.slice();
            }
            position = this.normalize(position, true);
            return [this._items[position]];
        };
        /**
         * Gets an item at the specified relative position.
         * @param position The relative position of the item.
         * @returns The item at the given position or all items if no position was given.
         */
        CarouselService.prototype.mergers = function (position) {
            if (position === undefined) {
                return this._mergers.slice();
            }
            position = this.normalize(position, true);
            return this._mergers[position];
        };
        /**
         * Gets the absolute positions of clones for an item.
         * @param position The relative position of the item.
         * @returns The absolute positions of clones for the item or all if no position was given.
         */
        CarouselService.prototype.clones = function (position) {
            var odd = this._clones.length / 2,
                even = odd + this._items.length,
                map = function (index) {
                    return index % 2 === 0 ? even + index / 2 : odd - (index + 1) / 2;
                };
            if (position === undefined) {
                return this._clones.map(function (v, i) {
                    return map(i);
                });
            }
            return this._clones
                .map(function (v, i) {
                    return v === position ? map(i) : null;
                })
                .filter(function (item) {
                    return item;
                });
        };
        /**
         * Sets the current animation speed.
         * @param speed The animation speed in milliseconds or nothing to leave it unchanged.
         * @returns The current animation speed in milliseconds.
         */
        CarouselService.prototype.speed = function (speed) {
            if (speed !== undefined) {
                this._speed = speed;
            }
            return this._speed;
        };
        /**
         * Gets the coordinate of an item.
         * @todo The name of this method is missleanding.
         * @param position The absolute position of the item within `minimum()` and `maximum()`.
         * @returns The coordinate of the item in pixel or all coordinates.
         */
        CarouselService.prototype.coordinates = function (position) {
            var _this = this;
            var multiplier = 1,
                newPosition = position - 1,
                coordinate,
                result;
            if (position === undefined) {
                result = this._coordinates.map(function (item, index) {
                    return _this.coordinates(index);
                });
                return result;
            }
            if (this.settings.center) {
                if (this.settings.rtl) {
                    multiplier = -1;
                    newPosition = position + 1;
                }
                coordinate = this._coordinates[position];
                coordinate += ((this.width() - coordinate + (this._coordinates[newPosition] || 0)) / 2) * multiplier;
            } else {
                coordinate = this._coordinates[newPosition] || 0;
            }
            coordinate = Math.ceil(coordinate);
            return coordinate;
        };
        /**
         * Calculates the speed for a translation.
         * @param from The absolute position of the start item.
         * @param to The absolute position of the target item.
         * @param factor [factor=undefined] - The time factor in milliseconds.
         * @returns The time in milliseconds for the translation.
         */
        CarouselService.prototype._duration = function (from, to, factor) {
            if (factor === 0) {
                return 0;
            }
            return Math.min(Math.max(Math.abs(to - from), 1), 6) * Math.abs(+factor || this.settings.smartSpeed);
        };
        /**
         * Slides to the specified item.
         * @param position The position of the item.
         * @param speed The time in milliseconds for the transition.
         */
        CarouselService.prototype.to = function (position, speed) {
            var _this = this;
            var current = this.current(),
                revert = null,
                distance = position - this.relative(current),
                maximum = this.maximum(),
                delayForLoop = 0;
            var direction = +(distance > 0) - +(distance < 0),
                items = this._items.length,
                minimum = this.minimum();
            if (this.settings.loop) {
                if (!this.settings.rewind && Math.abs(distance) > items / 2) {
                    distance += direction * -1 * items;
                }
                position = current + distance;
                revert = ((((position - minimum) % items) + items) % items) + minimum;
                if (revert !== position && revert - distance <= maximum && revert - distance > 0) {
                    current = revert - distance;
                    position = revert;
                    delayForLoop = 30;
                    this.reset(current);
                    this.sendChanges();
                }
            } else if (this.settings.rewind) {
                maximum += 1;
                position = ((position % maximum) + maximum) % maximum;
            } else {
                position = Math.max(minimum, Math.min(maximum, position));
            }
            setTimeout(function () {
                _this.speed(_this._duration(current, position, speed));
                _this.current(position);
                _this.update();
            }, delayForLoop);
        };
        /**
         * Slides to the next item.
         * @param speed The time in milliseconds for the transition.
         */
        CarouselService.prototype.next = function (speed) {
            speed = speed || false;
            this.to(this.relative(this.current()) + 1, speed);
        };
        /**
         * Slides to the previous item.
         * @param speed The time in milliseconds for the transition.
         */
        CarouselService.prototype.prev = function (speed) {
            speed = speed || false;
            this.to(this.relative(this.current()) - 1, speed);
        };
        /**
         * Handles the end of an animation.
         * @param event - The event arguments.
         */
        CarouselService.prototype.onTransitionEnd = function (event) {
            // if css2 animation then event object is undefined
            if (event !== undefined) {
                // event.stopPropagation();
                // // Catch only owl-stage transitionEnd event
                // if ((event.target || event.srcElement || event.originalTarget) !== this.$stage.get(0)	) {
                // 	return false;
                // }
                return false;
            }
            this.leave('animating');
            this._trigger('translated');
        };
        /**
         * Gets viewport width.
         * @returns - The width in pixel.
         */
        CarouselService.prototype._viewport = function () {
            var width;
            if (this._width) {
                width = this._width;
            } else {
                this.logger.log('Can not detect viewport width.');
            }
            return width;
        };
        /**
         * Sets _items
         * @param content The list of slides put into CarouselSlideDirectives.
         */
        CarouselService.prototype.setItems = function (content) {
            this._items = content;
        };
        /**
         * Sets slidesData using this._items
         */
        CarouselService.prototype._defineSlidesData = function () {
            // Maybe creating and using loadMap would be better in LazyLoadService.
            // Hovewer in that case when 'resize' event fires, prop 'load' of all slides will get 'false' and such state of prop will be seen by View during its updating. Accordingly the code will remove slides's content from DOM even if it was loaded before.
            // Thus it would be needed to add that content into DOM again.
            // In order to avoid additional removing/adding loaded slides's content we use loadMap here and set restore state of prop 'load' before the View will get it.
            var loadMap;
            if (this.slidesData && this.slidesData.length) {
                loadMap = new Map();
                this.slidesData.forEach(function (item) {
                    if (item.load) {
                        loadMap.set(item.id, item.load);
                    }
                });
            }
            this.slidesData = this._items.map(function (slide) {
                return {
                    id: '' + slide.id,
                    isActive: false,
                    tplRef: slide.tplRef,
                    dataMerge: slide.dataMerge,
                    width: 0,
                    isCloned: false,
                    load: loadMap ? loadMap.get(slide.id) : false,
                    hashFragment: slide.dataHash,
                };
            });
        };
        /**
         * Sets current classes for slide
         * @param slide Slide of carousel
         * @returns object with names of css-classes which are keys and true/false values
         */
        CarouselService.prototype.setCurSlideClasses = function (slide) {
            // CSS classes: added/removed per current state of component properties
            var currentClasses = {
                active: slide.isActive,
                center: slide.isCentered,
                cloned: slide.isCloned,
                animated: slide.isAnimated,
                'owl-animated-in': slide.isDefAnimatedIn,
                'owl-animated-out': slide.isDefAnimatedOut,
            };
            if (this.settings.animateIn) {
                currentClasses[this.settings.animateIn] = slide.isCustomAnimatedIn;
            }
            if (this.settings.animateOut) {
                currentClasses[this.settings.animateOut] = slide.isCustomAnimatedOut;
            }
            return currentClasses;
        };
        /**
         * Operators to calculate right-to-left and left-to-right.
         * @param a - The left side operand.
         * @param o - The operator.
         * @param b - The right side operand.
         * @returns true/false meaning right-to-left or left-to-right
         */
        CarouselService.prototype._op = function (a, o, b) {
            var rtl = this.settings.rtl;
            switch (o) {
                case '<':
                    return rtl ? a > b : a < b;
                case '>':
                    return rtl ? a < b : a > b;
                case '>=':
                    return rtl ? a <= b : a >= b;
                case '<=':
                    return rtl ? a >= b : a <= b;
                default:
                    break;
            }
        };
        /**
         * Triggers a public event.
         * @todo Remove `status`, `relatedTarget` should be used instead.
         * @param name The event name.
         * @param data The event data.
         * @param namespace The event namespace.
         * @param state The state which is associated with the event.
         * @param enter Indicates if the call enters the specified state or not.
         */
        CarouselService.prototype._trigger = function (name, data, namespace, state, enter) {
            switch (name) {
                case 'initialized':
                    this._initializedCarousel$.next(name);
                    break;
                case 'change':
                    this._changeSettingsCarousel$.next(data);
                    break;
                case 'changed':
                    this._changedSettingsCarousel$.next(data);
                    break;
                case 'drag':
                    this._dragCarousel$.next(name);
                    break;
                case 'dragged':
                    this._draggedCarousel$.next(name);
                    break;
                case 'resize':
                    this._resizeCarousel$.next(name);
                    break;
                case 'resized':
                    this._resizedCarousel$.next(name);
                    break;
                case 'refresh':
                    this._refreshCarousel$.next(name);
                    break;
                case 'refreshed':
                    this._refreshedCarousel$.next(name);
                    break;
                case 'translate':
                    this._translateCarousel$.next(name);
                    break;
                case 'translated':
                    this._translatedCarousel$.next(name);
                    break;
                default:
                    break;
            }
        };
        /**
         * Enters a state.
         * @param name - The state name.
         */
        CarouselService.prototype.enter = function (name) {
            var _this = this;
            [name].concat(this._states.tags[name] || []).forEach(function (stateName) {
                if (_this._states.current[stateName] === undefined) {
                    _this._states.current[stateName] = 0;
                }
                _this._states.current[stateName]++;
            });
        };
        /**
         * Leaves a state.
         * @param name - The state name.
         */
        CarouselService.prototype.leave = function (name) {
            var _this = this;
            [name].concat(this._states.tags[name] || []).forEach(function (stateName) {
                if (_this._states.current[stateName] === 0 || !!_this._states.current[stateName]) {
                    _this._states.current[stateName]--;
                }
            });
        };
        /**
         * Registers an event or state.
         * @param object - The event or state to register.
         */
        CarouselService.prototype.register = function (object) {
            var _this = this;
            if (object.type === Type.State) {
                if (!this._states.tags[object.name]) {
                    this._states.tags[object.name] = object.tags;
                } else {
                    this._states.tags[object.name] = this._states.tags[object.name].concat(object.tags);
                }
                this._states.tags[object.name] = this._states.tags[object.name].filter(function (tag, i) {
                    return _this._states.tags[object.name].indexOf(tag) === i;
                });
            }
        };
        /**
         * Suppresses events.
         * @param events The events to suppress.
         */
        CarouselService.prototype._suppress = function (events) {
            var _this = this;
            events.forEach(function (event) {
                _this._supress[event] = true;
            });
        };
        /**
         * Releases suppressed events.
         * @param events The events to release.
         */
        CarouselService.prototype._release = function (events) {
            var _this = this;
            events.forEach(function (event) {
                delete _this._supress[event];
            });
        };
        /**
         * Gets unified pointer coordinates from event.
         * @todo #261
         * @param event The `mousedown` or `touchstart` event.
         * @returns Object Coords which contains `x` and `y` coordinates of current pointer position.
         */
        CarouselService.prototype.pointer = function (event) {
            var result = { x: null, y: null };
            event = event.originalEvent || event || window.event;
            event =
                event.touches && event.touches.length
                    ? event.touches[0]
                    : event.changedTouches && event.changedTouches.length
                    ? event.changedTouches[0]
                    : event;
            if (event.pageX) {
                result.x = event.pageX;
                result.y = event.pageY;
            } else {
                result.x = event.clientX;
                result.y = event.clientY;
            }
            return result;
        };
        /**
         * Determines if the input is a Number or something that can be coerced to a Number
         * @param number The input to be tested
         * @returns An indication if the input is a Number or can be coerced to a Number
         */
        CarouselService.prototype._isNumeric = function (number) {
            return !isNaN(parseFloat(number));
        };
        /**
         * Determines whether value is number or boolean type
         * @param value The input to be tested
         * @returns An indication if the input is a Number or can be coerced to a Number, or Boolean
         */
        CarouselService.prototype._isNumberOrBoolean = function (value) {
            return this._isNumeric(value) || typeof value === 'boolean';
        };
        /**
         * Determines whether value is number or string type
         * @param value The input to be tested
         * @returns An indication if the input is a Number or can be coerced to a Number, or String
         */
        CarouselService.prototype._isNumberOrString = function (value) {
            return this._isNumeric(value) || typeof value === 'string';
        };
        /**
         * Determines whether value is number or string type
         * @param value The input to be tested
         * @returns An indication if the input is a Number or can be coerced to a Number, or String
         */
        CarouselService.prototype._isStringOrBoolean = function (value) {
            return typeof value === 'string' || typeof value === 'boolean';
        };
        /**
         * Gets the difference of two vectors.
         * @todo #261
         * @param first The first vector.
         * @param second- The second vector.
         * @returns The difference.
         */
        CarouselService.prototype.difference = function (first, second) {
            return {
                x: first.x - second.x,
                y: first.y - second.y,
            };
        };
        return CarouselService;
    })();
    CarouselService.decorators = [{ type: core.Injectable }];
    CarouselService.ctorParameters = function () {
        return [{ type: OwlLogger }];
    };

    var NavigationService = /** @class */ (function () {
        function NavigationService(carouselService) {
            this.carouselService = carouselService;
            /**
             * Indicates whether the plugin is initialized or not.
             */
            this._initialized = false;
            /**
             * The current paging indexes.
             */
            this._pages = [];
            /**
             * Data for navigation elements of the user interface.
             */
            this._navData = {
                disabled: false,
                prev: {
                    disabled: false,
                    htmlText: '',
                },
                next: {
                    disabled: false,
                    htmlText: '',
                },
            };
            /**
             * Data for dot elements of the user interface.
             */
            this._dotsData = {
                disabled: false,
                dots: [],
            };
            this.spyDataStreams();
        }
        NavigationService.prototype.ngOnDestroy = function () {
            this.navSubscription.unsubscribe();
        };
        /**
         * Defines Observables which service must observe
         */
        NavigationService.prototype.spyDataStreams = function () {
            var _this = this;
            var initializedCarousel$ = this.carouselService.getInitializedState().pipe(
                operators.tap(function (state) {
                    _this.initialize();
                    _this._updateNavPages();
                    _this.draw();
                    _this.update();
                    _this.carouselService.sendChanges();
                }),
            );
            // mostly changes in carouselService and carousel at all causes carouselService.to(). It moves stage right-left by its code and calling needed functions
            // Thus this method by calling carouselService.current(position) notifies about changes
            var changedSettings$ = this.carouselService.getChangedState().pipe(
                operators.filter(function (data) {
                    return data.property.name === 'position';
                }),
                operators.tap(function (data) {
                    _this.update();
                    // should be the call of the function written at the end of comment
                    // but the method carouselServive.to() has setTimeout(f, 0) which contains carouselServive.update() which calls sendChanges() method.
                    // carouselService.navData and carouselService.dotsData update earlier than carouselServive.update() gets called
                    // updates of carouselService.navData and carouselService.dotsData are being happening withing carouselService.current(position) method which calls next() of _changedSettingsCarousel$
                    // carouselService.current(position) is being calling earlier than carouselServive.update();
                    // this.carouselService.sendChanges();
                }),
            );
            var refreshedCarousel$ = this.carouselService.getRefreshedState().pipe(
                operators.tap(function () {
                    _this._updateNavPages();
                    _this.draw();
                    _this.update();
                    _this.carouselService.sendChanges();
                }),
            );
            var navMerge$ = rxjs.merge(initializedCarousel$, changedSettings$, refreshedCarousel$);
            this.navSubscription = navMerge$.subscribe(function () {});
        };
        /**
         * Initializes the layout of the plugin and extends the carousel.
         */
        NavigationService.prototype.initialize = function () {
            this._navData.disabled = true;
            this._navData.prev.htmlText = this.carouselService.settings.navText[0];
            this._navData.next.htmlText = this.carouselService.settings.navText[1];
            this._dotsData.disabled = true;
            this.carouselService.navData = this._navData;
            this.carouselService.dotsData = this._dotsData;
        };
        /**
         * Calculates internal states and updates prop _pages
         */
        NavigationService.prototype._updateNavPages = function () {
            var i, j, k;
            var lower = this.carouselService.clones().length / 2,
                upper = lower + this.carouselService.items().length,
                maximum = this.carouselService.maximum(true),
                pages = [],
                settings = this.carouselService.settings;
            var size = settings.center || settings.autoWidth || settings.dotsData ? 1 : settings.dotsEach || settings.items;
            size = +size;
            if (settings.slideBy !== 'page') {
                settings.slideBy = Math.min(+settings.slideBy, settings.items);
            }
            if (settings.dots || settings.slideBy === 'page') {
                for (i = lower, j = 0, k = 0; i < upper; i++) {
                    if (j >= size || j === 0) {
                        pages.push({
                            start: Math.min(maximum, i - lower),
                            end: i - lower + size - 1,
                        });
                        if (Math.min(maximum, i - lower) === maximum) {
                            break;
                        }
                        (j = 0), ++k;
                    }
                    j += this.carouselService.mergers(this.carouselService.relative(i));
                }
            }
            this._pages = pages;
        };
        /**
         * Draws the user interface.
         * @todo The option `dotsData` wont work.
         */
        NavigationService.prototype.draw = function () {
            var _this = this;
            var difference;
            var settings = this.carouselService.settings,
                items = this.carouselService.items(),
                disabled = items.length <= settings.items;
            this._navData.disabled = !settings.nav || disabled;
            this._dotsData.disabled = !settings.dots || disabled;
            if (settings.dots) {
                difference = this._pages.length - this._dotsData.dots.length;
                if (settings.dotsData && difference !== 0) {
                    this._dotsData.dots = [];
                    items.forEach(function (item) {
                        _this._dotsData.dots.push({
                            active: false,
                            id: 'dot-' + item.id,
                            innerContent: item.dotContent,
                            showInnerContent: true,
                        });
                    });
                } else if (difference > 0) {
                    var startI = this._dotsData.dots.length > 0 ? this._dotsData.dots.length : 0;
                    for (var i = 0; i < difference; i++) {
                        this._dotsData.dots.push({
                            active: false,
                            id: 'dot-' + (i + startI),
                            innerContent: '',
                            showInnerContent: false,
                        });
                    }
                } else if (difference < 0) {
                    this._dotsData.dots.splice(difference, Math.abs(difference));
                }
            }
            this.carouselService.navData = this._navData;
            this.carouselService.dotsData = this._dotsData;
        };
        /**
         * Updates navigation buttons's and dots's states
         */
        NavigationService.prototype.update = function () {
            this._updateNavButtons();
            this._updateDots();
        };
        /**
         * Changes state of nav buttons (disabled, enabled)
         */
        NavigationService.prototype._updateNavButtons = function () {
            var settings = this.carouselService.settings,
                loop = settings.loop || settings.rewind,
                index = this.carouselService.relative(this.carouselService.current());
            if (settings.nav) {
                this._navData.prev.disabled = !loop && index <= this.carouselService.minimum(true);
                this._navData.next.disabled = !loop && index >= this.carouselService.maximum(true);
            }
            this.carouselService.navData = this._navData;
        };
        /**
         * Changes active dot if page becomes changed
         */
        NavigationService.prototype._updateDots = function () {
            var curActiveDotI;
            if (!this.carouselService.settings.dots) {
                return;
            }
            this._dotsData.dots.forEach(function (item) {
                if (item.active === true) {
                    item.active = false;
                }
            });
            curActiveDotI = this._current();
            if (this._dotsData.dots.length) {
                this._dotsData.dots[curActiveDotI].active = true;
            }
            this.carouselService.dotsData = this._dotsData;
        };
        /**
         * Gets the current page position of the carousel.
         * @returns the current page position of the carousel
         */
        NavigationService.prototype._current = function () {
            var current = this.carouselService.relative(this.carouselService.current());
            var finalCurrent;
            var pages = this._pages
                .filter(function (page, index) {
                    return page.start <= current && page.end >= current;
                })
                .pop();
            finalCurrent = this._pages.findIndex(function (page) {
                return page.start === pages.start && page.end === pages.end;
            });
            return finalCurrent;
        };
        /**
         * Gets the current succesor/predecessor position.
         * @param sussessor position of slide
         * @returns the current succesor/predecessor position
         */
        NavigationService.prototype._getPosition = function (successor) {
            var position, length;
            var settings = this.carouselService.settings;
            if (settings.slideBy === 'page') {
                position = this._current();
                length = this._pages.length;
                successor ? ++position : --position;
                position = this._pages[((position % length) + length) % length].start;
            } else {
                position = this.carouselService.relative(this.carouselService.current());
                length = this.carouselService.items().length;
                successor ? (position += +settings.slideBy) : (position -= +settings.slideBy);
            }
            return position;
        };
        /**
         * Slides to the next item or page.
         * @param speed The time in milliseconds for the transition.
         */
        NavigationService.prototype.next = function (speed) {
            this.carouselService.to(this._getPosition(true), speed);
        };
        /**
         * Slides to the previous item or page.
         * @param speed The time in milliseconds for the transition.
         */
        NavigationService.prototype.prev = function (speed) {
            this.carouselService.to(this._getPosition(false), speed);
        };
        /**
         * Slides to the specified item or page.
         * @param position - The position of the item or page.
         * @param speed - The time in milliseconds for the transition.
         * @param standard - Whether to use the standard behaviour or not. Default meaning false
         */
        NavigationService.prototype.to = function (position, speed, standard) {
            var length;
            if (!standard && this._pages.length) {
                length = this._pages.length;
                this.carouselService.to(this._pages[((position % length) + length) % length].start, speed);
            } else {
                this.carouselService.to(position, speed);
            }
        };
        /**
         * Moves carousel after user's clicking on any dots
         */
        NavigationService.prototype.moveByDot = function (dotId) {
            var index = this._dotsData.dots.findIndex(function (dot) {
                return dotId === dot.id;
            });
            this.to(index, this.carouselService.settings.dotsSpeed);
        };
        /**
         * rewinds carousel to slide with needed id
         * @param id id of slide
         */
        NavigationService.prototype.toSlideById = function (id) {
            var position = this.carouselService.slidesData.findIndex(function (slide) {
                return slide.id === id && slide.isCloned === false;
            });
            if (position === -1 || position === this.carouselService.current()) {
                return;
            }
            this.carouselService.to(this.carouselService.relative(position), false);
        };
        return NavigationService;
    })();
    NavigationService.decorators = [{ type: core.Injectable }];
    NavigationService.ctorParameters = function () {
        return [{ type: CarouselService }];
    };

    /**
     * Create a new injection token for injecting the window into a component.
     */
    var WINDOW = new core.InjectionToken('WindowToken');
    /**
     * Define abstract class for obtaining reference to the global window object.
     */
    var WindowRef = /** @class */ (function () {
        function WindowRef() {}
        Object.defineProperty(WindowRef.prototype, 'nativeWindow', {
            get: function () {
                throw new Error('Not implemented.');
            },
            enumerable: false,
            configurable: true,
        });
        return WindowRef;
    })();
    /**
     * Define class that implements the abstract class and returns the native window object.
     */
    var BrowserWindowRef = /** @class */ (function (_super) {
        __extends(BrowserWindowRef, _super);
        function BrowserWindowRef() {
            return _super.call(this) || this;
        }
        Object.defineProperty(BrowserWindowRef.prototype, 'nativeWindow', {
            /**
             * @returns window object
             */
            get: function () {
                return window;
            },
            enumerable: false,
            configurable: true,
        });
        return BrowserWindowRef;
    })(WindowRef);
    BrowserWindowRef.decorators = [{ type: core.Injectable }];
    BrowserWindowRef.ctorParameters = function () {
        return [];
    };
    /**
     * Create an factory function that returns the native window object.
     * @param browserWindowRef Native window object
     * @param platformId id of platform
     * @returns type of platform of empty object
     */
    function windowFactory(browserWindowRef, platformId) {
        if (common.isPlatformBrowser(platformId)) {
            return browserWindowRef.nativeWindow;
        }
        var obj = {
            setTimeout: function (func, time) {},
            clearTimeout: function (a) {},
        };
        return obj;
    }
    /**
     * Create a injectable provider for the WindowRef token that uses the BrowserWindowRef class.
     */
    var browserWindowProvider = {
        provide: WindowRef,
        useClass: BrowserWindowRef,
    };
    /**
     * Create an injectable provider that uses the windowFactory function for returning the native window object.
     */
    var windowProvider = {
        provide: WINDOW,
        useFactory: windowFactory,
        deps: [WindowRef, core.PLATFORM_ID],
    };
    /**
     * Create an array of providers.
     */
    var WINDOW_PROVIDERS = [browserWindowProvider, windowProvider];

    /**
     * Create a new injection token for injecting the Document into a component.
     */
    var DOCUMENT = new core.InjectionToken('DocumentToken');
    /**
     * Define abstract class for obtaining reference to the global Document object.
     */
    var DocumentRef = /** @class */ (function () {
        function DocumentRef() {}
        Object.defineProperty(DocumentRef.prototype, 'nativeDocument', {
            get: function () {
                throw new Error('Not implemented.');
            },
            enumerable: false,
            configurable: true,
        });
        return DocumentRef;
    })();
    /**
     * Define class that implements the abstract class and returns the native Document object.
     */
    var BrowserDocumentRef = /** @class */ (function (_super) {
        __extends(BrowserDocumentRef, _super);
        function BrowserDocumentRef() {
            return _super.call(this) || this;
        }
        Object.defineProperty(BrowserDocumentRef.prototype, 'nativeDocument', {
            /**
             * @returns Document object
             */
            get: function () {
                return document;
            },
            enumerable: false,
            configurable: true,
        });
        return BrowserDocumentRef;
    })(DocumentRef);
    BrowserDocumentRef.decorators = [{ type: core.Injectable }];
    BrowserDocumentRef.ctorParameters = function () {
        return [];
    };
    /**
     * Create an factory function that returns the native Document object.
     * @param browserDocumentRef Native Document object
     * @param platformId id of platform
     * @returns type of platform of empty object
     */
    function documentFactory(browserDocumentRef, platformId) {
        if (common.isPlatformBrowser(platformId)) {
            return browserDocumentRef.nativeDocument;
        }
        var doc = {
            hidden: false,
            visibilityState: 'visible',
        };
        return doc;
    }
    /**
     * Create a injectable provider for the DocumentRef token that uses the BrowserDocumentRef class.
     */
    var browserDocumentProvider = {
        provide: DocumentRef,
        useClass: BrowserDocumentRef,
    };
    /**
     * Create an injectable provider that uses the DocumentFactory function for returning the native Document object.
     */
    var documentProvider = {
        provide: DOCUMENT,
        useFactory: documentFactory,
        deps: [DocumentRef, core.PLATFORM_ID],
    };
    /**
     * Create an array of providers.
     */
    var DOCUMENT_PROVIDERS = [browserDocumentProvider, documentProvider];

    var AutoplayService = /** @class */ (function () {
        function AutoplayService(carouselService, winRef, docRef) {
            this.carouselService = carouselService;
            /**
             * The autoplay timeout.
             */
            this._timeout = null;
            /**
             * Indicates whenever the autoplay is paused.
             */
            this._paused = false;
            this.winRef = winRef;
            this.docRef = docRef;
            this.spyDataStreams();
        }
        AutoplayService.prototype.ngOnDestroy = function () {
            this.autoplaySubscription.unsubscribe();
        };
        /**
         * Defines Observables which service must observe
         */
        AutoplayService.prototype.spyDataStreams = function () {
            var _this = this;
            var initializedCarousel$ = this.carouselService.getInitializedState().pipe(
                operators.tap(function () {
                    if (_this.carouselService.settings.autoplay) {
                        _this.play();
                    }
                }),
            );
            var changedSettings$ = this.carouselService.getChangedState().pipe(
                operators.tap(function (data) {
                    _this._handleChangeObservable(data);
                }),
            );
            var resized$ = this.carouselService.getResizedState().pipe(
                operators.tap(function () {
                    if (_this.carouselService.settings.autoplay) {
                        _this.play();
                    } else {
                        _this.stop();
                    }
                }),
            );
            // original Autoplay Plugin has listeners on play.owl.core and stop.owl.core events.
            // They are triggered by Video Plugin
            var autoplayMerge$ = rxjs.merge(initializedCarousel$, changedSettings$, resized$);
            this.autoplaySubscription = autoplayMerge$.subscribe(function () {});
        };
        /**
         * Starts the autoplay.
         * @param timeout The interval before the next animation starts.
         * @param speed The animation speed for the animations.
         */
        AutoplayService.prototype.play = function (timeout, speed) {
            if (this._paused) {
                this._paused = false;
                this._setAutoPlayInterval(this.carouselService.settings.autoplayMouseleaveTimeout);
            }
            if (this.carouselService.is('rotating')) {
                return;
            }
            this.carouselService.enter('rotating');
            this._setAutoPlayInterval();
        };
        /**
         * Gets a new timeout
         * @param timeout - The interval before the next animation starts.
         * @param speed - The animation speed for the animations.
         * @return
         */
        AutoplayService.prototype._getNextTimeout = function (timeout, speed) {
            var _this = this;
            if (this._timeout) {
                this.winRef.clearTimeout(this._timeout);
            }
            this._isArtificialAutoplayTimeout = timeout ? true : false;
            return this.winRef.setTimeout(function () {
                if (_this._paused || _this.carouselService.is('busy') || _this.carouselService.is('interacting') || _this.docRef.hidden) {
                    return;
                }
                _this.carouselService.next(speed || _this.carouselService.settings.autoplaySpeed);
            }, timeout || this.carouselService.settings.autoplayTimeout);
        };
        /**
         * Sets autoplay in motion.
         */
        AutoplayService.prototype._setAutoPlayInterval = function (timeout) {
            this._timeout = this._getNextTimeout(timeout);
        };
        /**
         * Stops the autoplay.
         */
        AutoplayService.prototype.stop = function () {
            if (!this.carouselService.is('rotating')) {
                return;
            }
            this._paused = true;
            this.winRef.clearTimeout(this._timeout);
            this.carouselService.leave('rotating');
        };
        /**
         * Stops the autoplay.
         */
        AutoplayService.prototype.pause = function () {
            if (!this.carouselService.is('rotating')) {
                return;
            }
            this._paused = true;
        };
        /**
         * Manages by autoplaying according to data passed by _changedSettingsCarousel$ Obsarvable
         * @param data object with current position of carousel and type of change
         */
        AutoplayService.prototype._handleChangeObservable = function (data) {
            if (data.property.name === 'settings') {
                if (this.carouselService.settings.autoplay) {
                    this.play();
                } else {
                    this.stop();
                }
            } else if (data.property.name === 'position') {
                //console.log('play?', e);
                if (this.carouselService.settings.autoplay) {
                    this._setAutoPlayInterval();
                }
            }
        };
        /**
         * Starts autoplaying of the carousel in the case when user leaves the carousel before it starts translateing (moving)
         */
        AutoplayService.prototype._playAfterTranslated = function () {
            var _this = this;
            rxjs.of('translated')
                .pipe(
                    operators.switchMap(function (data) {
                        return _this.carouselService.getTranslatedState();
                    }),
                    operators.first(),
                    operators.filter(function () {
                        return _this._isArtificialAutoplayTimeout;
                    }),
                    operators.tap(function () {
                        return _this._setAutoPlayInterval();
                    }),
                )
                .subscribe(function () {});
        };
        /**
         * Starts pausing
         */
        AutoplayService.prototype.startPausing = function () {
            if (this.carouselService.settings.autoplayHoverPause && this.carouselService.is('rotating')) {
                this.pause();
            }
        };
        /**
         * Starts playing after mouse leaves carousel
         */
        AutoplayService.prototype.startPlayingMouseLeave = function () {
            if (this.carouselService.settings.autoplayHoverPause && this.carouselService.is('rotating')) {
                this.play();
                this._playAfterTranslated();
            }
        };
        /**
         * Starts playing after touch ends
         */
        AutoplayService.prototype.startPlayingTouchEnd = function () {
            if (this.carouselService.settings.autoplayHoverPause && this.carouselService.is('rotating')) {
                this.play();
                this._playAfterTranslated();
            }
        };
        return AutoplayService;
    })();
    AutoplayService.decorators = [{ type: core.Injectable }];
    AutoplayService.ctorParameters = function () {
        return [
            { type: CarouselService },
            { type: undefined, decorators: [{ type: core.Inject, args: [WINDOW] }] },
            { type: undefined, decorators: [{ type: core.Inject, args: [DOCUMENT] }] },
        ];
    };

    var LazyLoadService = /** @class */ (function () {
        function LazyLoadService(carouselService) {
            this.carouselService = carouselService;
            this.spyDataStreams();
        }
        LazyLoadService.prototype.ngOnDestroy = function () {
            this.lazyLoadSubscription.unsubscribe();
        };
        /**
         * Defines Observables which service must observe
         */
        LazyLoadService.prototype.spyDataStreams = function () {
            var _this = this;
            var initializedCarousel$ = this.carouselService.getInitializedState().pipe(
                operators.tap(function () {
                    var isLazyLoad = _this.carouselService.settings && !_this.carouselService.settings.lazyLoad;
                    _this.carouselService.slidesData.forEach(function (item) {
                        return (item.load = isLazyLoad ? true : false);
                    });
                }),
            );
            var changeSettings$ = this.carouselService.getChangeState();
            var resizedCarousel$ = this.carouselService.getResizedState();
            var lazyLoadMerge$ = rxjs.merge(initializedCarousel$, changeSettings$, resizedCarousel$).pipe(
                operators.tap(function (data) {
                    return _this._defineLazyLoadSlides(data);
                }),
            );
            this.lazyLoadSubscription = lazyLoadMerge$.subscribe(function () {});
        };
        LazyLoadService.prototype._defineLazyLoadSlides = function (data) {
            var _this = this;
            if (!this.carouselService.settings || !this.carouselService.settings.lazyLoad) {
                return;
            }
            if ((data.property && data.property.name === 'position') || data === 'initialized' || data === 'resized') {
                var settings = this.carouselService.settings,
                    clones = this.carouselService.clones().length;
                var n = (settings.center && Math.ceil(settings.items / 2)) || settings.items,
                    i = (settings.center && n * -1) || 0,
                    position = (data.property && data.property.value !== undefined ? data.property.value : this.carouselService.current()) + i;
                // load = $.proxy(function(i, v) { this.load(v) }, this);
                //TODO: Need documentation for this new option
                if (settings.lazyLoadEager > 0) {
                    n += settings.lazyLoadEager;
                    // If the carousel is looping also preload images that are to the "left"
                    if (settings.loop) {
                        position -= settings.lazyLoadEager;
                        n++;
                    }
                }
                while (i++ < n) {
                    this._load(clones / 2 + this.carouselService.relative(position));
                    if (clones) {
                        this.carouselService.clones(this.carouselService.relative(position)).forEach(function (value) {
                            return _this._load(value);
                        });
                    }
                    position++;
                }
            }
        };
        /**
         * Loads all resources of an item at the specified position.
         * @param position - The absolute position of the item.
         */
        LazyLoadService.prototype._load = function (position) {
            if (this.carouselService.slidesData[position].load) {
                return;
            }
            this.carouselService.slidesData[position].load = true;
        };
        return LazyLoadService;
    })();
    LazyLoadService.decorators = [{ type: core.Injectable }];
    LazyLoadService.ctorParameters = function () {
        return [{ type: CarouselService }];
    };

    var AnimateService = /** @class */ (function () {
        function AnimateService(carouselService) {
            this.carouselService = carouselService;
            /**
             * s
             */
            this.swapping = true;
            /**
             * active slide before translating
             */
            this.previous = undefined;
            /**
             * new active slide after translating
             */
            this.next = undefined;
            this.spyDataStreams();
        }
        AnimateService.prototype.ngOnDestroy = function () {
            this.animateSubscription.unsubscribe();
        };
        /**
         * Defines Observables which service must observe
         */
        AnimateService.prototype.spyDataStreams = function () {
            var _this = this;
            var changeSettings$ = this.carouselService.getChangeState().pipe(
                operators.tap(function (data) {
                    if (data.property.name === 'position') {
                        _this.previous = _this.carouselService.current();
                        _this.next = data.property.value;
                    }
                }),
            );
            var dragCarousel$ = this.carouselService.getDragState();
            var draggedCarousel$ = this.carouselService.getDraggedState();
            var translatedCarousel$ = this.carouselService.getTranslatedState();
            var dragTranslatedMerge$ = rxjs.merge(dragCarousel$, draggedCarousel$, translatedCarousel$).pipe(
                operators.tap(function (data) {
                    return (_this.swapping = data === 'translated');
                }),
            );
            var translateCarousel$ = this.carouselService.getTranslateState().pipe(
                operators.tap(function (data) {
                    if (_this.swapping && (_this.carouselService._options.animateOut || _this.carouselService._options.animateIn)) {
                        _this._swap();
                    }
                }),
            );
            var animateMerge$ = rxjs.merge(changeSettings$, translateCarousel$, dragTranslatedMerge$).pipe();
            this.animateSubscription = animateMerge$.subscribe(function () {});
        };
        /**
         * Toggles the animation classes whenever an translations starts.
         * @returns
         */
        AnimateService.prototype._swap = function () {
            if (this.carouselService.settings.items !== 1) {
                return;
            }
            // if (!$.support.animation || !$.support.transition) {
            // 	return;
            // }
            this.carouselService.speed(0);
            var left;
            var previous = this.carouselService.slidesData[this.previous],
                next = this.carouselService.slidesData[this.next],
                incoming = this.carouselService.settings.animateIn,
                outgoing = this.carouselService.settings.animateOut;
            if (this.carouselService.current() === this.previous) {
                return;
            }
            if (outgoing) {
                left = +this.carouselService.coordinates(this.previous) - +this.carouselService.coordinates(this.next);
                this.carouselService.slidesData.forEach(function (slide) {
                    if (slide.id === previous.id) {
                        slide.left = left + 'px';
                        slide.isAnimated = true;
                        slide.isDefAnimatedOut = true;
                        slide.isCustomAnimatedOut = true;
                    }
                });
            }
            if (incoming) {
                this.carouselService.slidesData.forEach(function (slide) {
                    if (slide.id === next.id) {
                        slide.isAnimated = true;
                        slide.isDefAnimatedIn = true;
                        slide.isCustomAnimatedIn = true;
                    }
                });
            }
        };
        /**
         * Handles the end of 'animationend' event
         * @param id Id of slides
         */
        AnimateService.prototype.clear = function (id) {
            var _this = this;
            this.carouselService.slidesData.forEach(function (slide) {
                if (slide.id === id) {
                    slide.left = '';
                    slide.isAnimated = false;
                    slide.isDefAnimatedOut = false;
                    slide.isCustomAnimatedOut = false;
                    slide.isDefAnimatedIn = false;
                    slide.isCustomAnimatedIn = false;
                    slide.classes = _this.carouselService.setCurSlideClasses(slide);
                }
            });
            this.carouselService.onTransitionEnd();
        };
        return AnimateService;
    })();
    AnimateService.decorators = [{ type: core.Injectable }];
    AnimateService.ctorParameters = function () {
        return [{ type: CarouselService }];
    };

    var AutoHeightService = /** @class */ (function () {
        function AutoHeightService(carouselService) {
            this.carouselService = carouselService;
            this.spyDataStreams();
        }
        AutoHeightService.prototype.ngOnDestroy = function () {
            this.autoHeightSubscription.unsubscribe();
        };
        /**
         * Defines Observables which service must observe
         */
        AutoHeightService.prototype.spyDataStreams = function () {
            var _this = this;
            var initializedCarousel$ = this.carouselService.getInitializedState().pipe(
                operators.tap(function (data) {
                    if (_this.carouselService.settings.autoHeight) {
                        _this.update();
                    } else {
                        _this.carouselService.slidesData.forEach(function (slide) {
                            return (slide.heightState = 'full');
                        });
                    }
                }),
            );
            var changedSettings$ = this.carouselService.getChangedState().pipe(
                operators.tap(function (data) {
                    if (_this.carouselService.settings.autoHeight && data.property.name === 'position') {
                        _this.update();
                    }
                }),
            );
            var refreshedCarousel$ = this.carouselService.getRefreshedState().pipe(
                operators.tap(function (data) {
                    if (_this.carouselService.settings.autoHeight) {
                        _this.update();
                    }
                }),
            );
            var autoHeight$ = rxjs.merge(initializedCarousel$, changedSettings$, refreshedCarousel$);
            this.autoHeightSubscription = autoHeight$.subscribe(function () {});
        };
        /**
         * Updates the prop 'heightState' of slides
         */
        AutoHeightService.prototype.update = function () {
            var items = this.carouselService.settings.items;
            var start = this.carouselService.current(),
                end = start + items;
            if (this.carouselService.settings.center) {
                start = items % 2 === 1 ? start - (items - 1) / 2 : start - items / 2;
                end = items % 2 === 1 ? start + items : start + items + 1;
            }
            this.carouselService.slidesData.forEach(function (slide, i) {
                slide.heightState = i >= start && i < end ? 'full' : 'nulled';
            });
        };
        return AutoHeightService;
    })();
    AutoHeightService.decorators = [{ type: core.Injectable }];
    AutoHeightService.ctorParameters = function () {
        return [{ type: CarouselService }];
    };

    var HashService = /** @class */ (function () {
        function HashService(carouselService, route, router) {
            this.carouselService = carouselService;
            this.route = route;
            this.router = router;
            this.spyDataStreams();
            if (!this.route) {
                this.route = {
                    fragment: rxjs.of('no route').pipe(operators.take(1)),
                };
            }
            if (!this.router) {
                this.router = {
                    navigate: function (commands, extras) {
                        return;
                    },
                };
            }
        }
        HashService.prototype.ngOnDestroy = function () {
            this.hashSubscription.unsubscribe();
        };
        /**
         * Defines Observables which service must observe
         */
        HashService.prototype.spyDataStreams = function () {
            var _this = this;
            var initializedCarousel$ = this.carouselService.getInitializedState().pipe(
                operators.tap(function () {
                    return _this.listenToRoute();
                }),
            );
            var changedSettings$ = this.carouselService.getChangedState().pipe(
                operators.tap(function (data) {
                    if (_this.carouselService.settings.URLhashListener && data.property.name === 'position') {
                        var newCurSlide = _this.carouselService.current();
                        var newCurFragment = _this.carouselService.slidesData[newCurSlide].hashFragment;
                        if (!newCurFragment || newCurFragment === _this.currentHashFragment) {
                            return;
                        }
                        _this.router.navigate(['./'], { fragment: newCurFragment, relativeTo: _this.route });
                    }
                }),
            );
            var hashFragment$ = rxjs.merge(initializedCarousel$, changedSettings$);
            this.hashSubscription = hashFragment$.subscribe(function () {});
        };
        /**
         * rewinds carousel to slide which has the same hashFragment as fragment of current url
         * @param fragment fragment of url
         */
        HashService.prototype.rewind = function (fragment) {
            var position = this.carouselService.slidesData.findIndex(function (slide) {
                return slide.hashFragment === fragment && slide.isCloned === false;
            });
            if (position === -1 || position === this.carouselService.current()) {
                return;
            }
            this.carouselService.to(this.carouselService.relative(position), false);
        };
        /**
         * Initiate listening to ActivatedRoute.fragment
         */
        HashService.prototype.listenToRoute = function () {
            var _this = this;
            var count = this.carouselService.settings.startPosition === 'URLHash' ? 0 : 2;
            this.route.fragment.pipe(operators.skip(count)).subscribe(function (fragment) {
                _this.currentHashFragment = fragment;
                _this.rewind(fragment);
            });
        };
        return HashService;
    })();
    HashService.decorators = [{ type: core.Injectable }];
    HashService.ctorParameters = function () {
        return [
            { type: CarouselService },
            { type: router.ActivatedRoute, decorators: [{ type: core.Optional }] },
            { type: router.Router, decorators: [{ type: core.Optional }] },
        ];
    };

    var nextId = 0;
    var CarouselSlideDirective = /** @class */ (function () {
        function CarouselSlideDirective(tplRef) {
            this.tplRef = tplRef;
            /**
             * Unique slide identifier. Must be unique for the entire document for proper accessibility support.
             * Will be auto-generated if not provided.
             */
            this.id = 'owl-slide-' + nextId++;
            /**
             * Defines how much widths of common slide will current slide have
             * e.g. if _mergeData=2, the slide will twice wider then slides with _mergeData=1
             */
            this._dataMerge = 1;
            /**
             * Width of slide
             */
            this.width = 0;
            /**
             * Inner content of dot for certain slide; can be html-markup
             */
            this.dotContent = '';
            /**
             * Hash (fragment) of url which corresponds to certain slide
             */
            this.dataHash = '';
        }
        Object.defineProperty(CarouselSlideDirective.prototype, 'dataMerge', {
            get: function () {
                return this._dataMerge;
            },
            set: function (data) {
                this._dataMerge = this.isNumeric(data) ? data : 1;
            },
            enumerable: false,
            configurable: true,
        });
        /**
         * Determines if the input is a Number or something that can be coerced to a Number
         * @param - The input to be tested
         * @returns - An indication if the input is a Number or can be coerced to a Number
         */
        CarouselSlideDirective.prototype.isNumeric = function (number) {
            return !isNaN(parseFloat(number));
        };
        return CarouselSlideDirective;
    })();
    CarouselSlideDirective.decorators = [{ type: core.Directive, args: [{ selector: 'ng-template[carouselSlide]' }] }];
    CarouselSlideDirective.ctorParameters = function () {
        return [{ type: core.TemplateRef }];
    };
    CarouselSlideDirective.propDecorators = {
        id: [{ type: core.Input }],
        dataMerge: [{ type: core.Input }],
        width: [{ type: core.Input }],
        dotContent: [{ type: core.Input }],
        dataHash: [{ type: core.Input }],
    };
    /**
     * Data which will be passed out after ending of transition of carousel
     */
    var SlidesOutputData = /** @class */ (function () {
        function SlidesOutputData() {}
        return SlidesOutputData;
    })();
    var CarouselComponent = /** @class */ (function () {
        function CarouselComponent(
            el,
            resizeService,
            carouselService,
            navigationService,
            autoplayService,
            lazyLoadService,
            animateService,
            autoHeightService,
            hashService,
            logger,
            changeDetectorRef,
            docRef,
        ) {
            this.el = el;
            this.resizeService = resizeService;
            this.carouselService = carouselService;
            this.navigationService = navigationService;
            this.autoplayService = autoplayService;
            this.lazyLoadService = lazyLoadService;
            this.animateService = animateService;
            this.autoHeightService = autoHeightService;
            this.hashService = hashService;
            this.logger = logger;
            this.changeDetectorRef = changeDetectorRef;
            this.translated = new core.EventEmitter();
            this.dragging = new core.EventEmitter();
            this.change = new core.EventEmitter();
            this.changed = new core.EventEmitter();
            this.initialized = new core.EventEmitter();
            /**
             *  Data of every slide
             */
            this.slidesData = [];
            /**
             * Shows whether carousel is loaded of not.
             */
            this.carouselLoaded = false;
            this.docRef = docRef;
        }
        CarouselComponent.prototype.onVisibilityChange = function (ev) {
            if (!this.carouselService.settings.autoplay) return;
            switch (this.docRef.visibilityState) {
                case 'visible':
                    this.autoplayService.play();
                    break;
                case 'hidden':
                    this.autoplayService.stop();
                    break;
                default:
                    break;
            }
        };
        CarouselComponent.prototype.ngOnInit = function () {
            this.spyDataStreams();
            this.carouselWindowWidth = this.el.nativeElement.querySelector('.owl-carousel').clientWidth;
        };
        CarouselComponent.prototype.ngAfterContentChecked = function () {};
        // ngAfterContentChecked() END
        CarouselComponent.prototype.ngAfterContentInit = function () {
            var _this = this;
            if (this.slides.toArray().length) {
                this.carouselService.setup(this.carouselWindowWidth, this.slides.toArray(), this.options);
                this.carouselService.initialize(this.slides.toArray());
                this._winResizeWatcher();
            } else {
                this.logger.log("There are no slides to show. So the carousel won't be rendered");
            }
            this._slidesChangesSubscription = this.slides.changes
                .pipe(
                    operators.tap(function (slides) {
                        if (slides.toArray().length) {
                            // this.carouselService.setItems(slides.toArray());
                            _this.carouselService.setup(_this.carouselWindowWidth, slides.toArray(), _this.options);
                            _this.carouselService.initialize(slides.toArray());
                        } else {
                            _this.carouselLoaded = false;
                            _this.logger.log("There are no slides to show. So the carousel won't be re-rendered");
                        }
                    }),
                )
                .subscribe(function () {});
        };
        CarouselComponent.prototype.ngOnDestroy = function () {
            if (this.resizeSubscription) {
                this.resizeSubscription.unsubscribe();
            }
            if (this._slidesChangesSubscription) {
                this._slidesChangesSubscription.unsubscribe();
            }
            if (this._allObservSubscription) {
                this._allObservSubscription.unsubscribe();
            }
        };
        /**
         * Joins the observable login in one place: sets values to some observables, merges this observables and
         * subcribes to merge func
         */
        CarouselComponent.prototype.spyDataStreams = function () {
            var _this = this;
            this._viewCurSettings$ = this.carouselService.getViewCurSettings().pipe(
                operators.tap(function (data) {
                    _this.owlDOMData = data.owlDOMData;
                    _this.stageData = data.stageData;
                    _this.slidesData = data.slidesData;
                    if (!_this.carouselLoaded) {
                        _this.carouselLoaded = true;
                    }
                    _this.navData = data.navData;
                    _this.dotsData = data.dotsData;
                    _this.changeDetectorRef.markForCheck();
                }),
            );
            this._initializedCarousel$ = this.carouselService.getInitializedState().pipe(
                operators.tap(function () {
                    _this.gatherTranslatedData();
                    _this.initialized.emit(_this.slidesOutputData);
                    // this.slidesOutputData = {};
                }),
            );
            this._translatedCarousel$ = this.carouselService.getTranslatedState().pipe(
                operators.tap(function () {
                    _this.gatherTranslatedData();
                    _this.translated.emit(_this.slidesOutputData);
                    // this.slidesOutputData = {};
                }),
            );
            this._changeCarousel$ = this.carouselService.getChangeState().pipe(
                operators.tap(function () {
                    _this.gatherTranslatedData();
                    _this.change.emit(_this.slidesOutputData);
                    // this.slidesOutputData = {};
                }),
            );
            this._changedCarousel$ = this.carouselService.getChangeState().pipe(
                operators.switchMap(function (value) {
                    var changedPosition = rxjs.of(value).pipe(
                        operators.filter(function () {
                            return value.property.name === 'position';
                        }),
                        operators.switchMap(function () {
                            return rxjs.from(_this.slidesData);
                        }),
                        operators.skip(value.property.value),
                        operators.take(_this.carouselService.settings.items),
                        operators.map(function (slide) {
                            var clonedIdPrefix = _this.carouselService.clonedIdPrefix;
                            var id = slide.id.indexOf(clonedIdPrefix) >= 0 ? slide.id.slice(clonedIdPrefix.length) : slide.id;
                            return Object.assign(Object.assign({}, slide), { id: id, isActive: true });
                        }),
                        operators.toArray(),
                        operators.map(function (slides) {
                            return {
                                slides: slides,
                                startPosition: _this.carouselService.relative(value.property.value),
                            };
                        }),
                    );
                    // const changedSetting: Observable<SlidesOutputData> = of(value).pipe(
                    //   filter(() => value.property.name === 'settings'),
                    //   map(() => {
                    //     return {
                    //       slides: [],
                    //       startPosition: this.carouselService.relative(value.property.value)
                    //     }
                    //   })
                    // )
                    return rxjs.merge(changedPosition);
                }),
                operators.tap(function (slidesData) {
                    _this.gatherTranslatedData();
                    _this.changed.emit(slidesData.slides.length ? slidesData : _this.slidesOutputData);
                    // console.log(this.slidesOutputData);
                    // this.slidesOutputData = {};
                }),
            );
            this._draggingCarousel$ = this.carouselService.getDragState().pipe(
                operators.tap(function () {
                    _this.gatherTranslatedData();
                    _this.dragging.emit({ dragging: true, data: _this.slidesOutputData });
                }),
                operators.switchMap(function () {
                    return _this.carouselService.getDraggedState().pipe(
                        operators.map(function () {
                            return !!_this.carouselService.is('animating');
                        }),
                    );
                }),
                operators.switchMap(function (anim) {
                    if (anim) {
                        return _this.carouselService.getTranslatedState().pipe(operators.first());
                    } else {
                        return rxjs.of('not animating');
                    }
                }),
                operators.tap(function () {
                    _this.dragging.emit({ dragging: false, data: _this.slidesOutputData });
                }),
            );
            this._carouselMerge$ = rxjs.merge(
                this._viewCurSettings$,
                this._translatedCarousel$,
                this._draggingCarousel$,
                this._changeCarousel$,
                this._changedCarousel$,
                this._initializedCarousel$,
            );
            this._allObservSubscription = this._carouselMerge$.subscribe(function () {});
        };
        /**
         * Init subscription to resize event and attaches handler for this event
         */
        CarouselComponent.prototype._winResizeWatcher = function () {
            var _this = this;
            if (Object.keys(this.carouselService._options.responsive).length) {
                this.resizeSubscription = this.resizeService.onResize$
                    .pipe(
                        operators.filter(function () {
                            return _this.carouselWindowWidth !== _this.el.nativeElement.querySelector('.owl-carousel').clientWidth;
                        }),
                        operators.delay(this.carouselService.settings.responsiveRefreshRate),
                    )
                    .subscribe(function () {
                        _this.carouselService.onResize(_this.el.nativeElement.querySelector('.owl-carousel').clientWidth);
                        _this.carouselWindowWidth = _this.el.nativeElement.querySelector('.owl-carousel').clientWidth;
                    });
            }
        };
        /**
         * Handler for transitioend event
         */
        CarouselComponent.prototype.onTransitionEnd = function () {
            this.carouselService.onTransitionEnd();
        };
        /**
         * Handler for click event, attached to next button
         */
        CarouselComponent.prototype.next = function () {
            if (!this.carouselLoaded) return;
            this.navigationService.next(this.carouselService.settings.navSpeed);
        };
        /**
         * Handler for click event, attached to prev button
         */
        CarouselComponent.prototype.prev = function () {
            if (!this.carouselLoaded) return;
            this.navigationService.prev(this.carouselService.settings.navSpeed);
        };
        /**
         * Handler for click event, attached to dots
         */
        CarouselComponent.prototype.moveByDot = function (dotId) {
            if (!this.carouselLoaded) return;
            this.navigationService.moveByDot(dotId);
        };
        /**
         * rewinds carousel to slide with needed id
         * @param id fragment of url
         */
        CarouselComponent.prototype.to = function (id) {
            // if (!this.carouselLoaded || ((this.navData && this.navData.disabled) && (this.dotsData && this.dotsData.disabled))) return;
            if (!this.carouselLoaded) return;
            this.navigationService.toSlideById(id);
        };
        /**
         * Gathers and prepares data intended for passing to the user by means of firing event translatedCarousel
         */
        CarouselComponent.prototype.gatherTranslatedData = function () {
            var startPosition;
            var clonedIdPrefix = this.carouselService.clonedIdPrefix;
            var activeSlides = this.slidesData
                .filter(function (slide) {
                    return slide.isActive === true;
                })
                .map(function (slide) {
                    var id = slide.id.indexOf(clonedIdPrefix) >= 0 ? slide.id.slice(clonedIdPrefix.length) : slide.id;
                    return {
                        id: id,
                        width: slide.width,
                        marginL: slide.marginL,
                        marginR: slide.marginR,
                        center: slide.isCentered,
                    };
                });
            startPosition = this.carouselService.relative(this.carouselService.current());
            this.slidesOutputData = {
                startPosition: startPosition,
                slides: activeSlides,
            };
        };
        /**
         * Starts pausing
         */
        CarouselComponent.prototype.startPausing = function () {
            this.autoplayService.startPausing();
        };
        /**
         * Starts playing after mouse leaves carousel
         */
        CarouselComponent.prototype.startPlayML = function () {
            this.autoplayService.startPlayingMouseLeave();
        };
        /**
         * Starts playing after touch ends
         */
        CarouselComponent.prototype.startPlayTE = function () {
            this.autoplayService.startPlayingTouchEnd();
        };
        return CarouselComponent;
    })();
    CarouselComponent.decorators = [
        {
            type: core.Component,
            args: [
                {
                    selector: 'owl-carousel-o',
                    template:
                        '\n    <div class="owl-carousel owl-theme" #owlCarousel\n      [ngClass]="{\'owl-rtl\': owlDOMData?.rtl,\n                  \'owl-loaded\': owlDOMData?.isLoaded,\n                  \'owl-responsive\': owlDOMData?.isResponsive,\n                  \'owl-drag\': owlDOMData?.isMouseDragable,\n                  \'owl-grab\': owlDOMData?.isGrab}"\n      (mouseover)="startPausing()"\n      (mouseleave)="startPlayML()"\n      (touchstart)="startPausing()"\n      (touchend)="startPlayTE()">\n\n      <div *ngIf="carouselLoaded" class="owl-stage-outer">\n        <owl-stage [owlDraggable]="{\'isMouseDragable\': owlDOMData?.isMouseDragable, \'isTouchDragable\': owlDOMData?.isTouchDragable}"\n                    [stageData]="stageData"\n                    [slidesData]="slidesData"></owl-stage>\n      </div> <!-- /.owl-stage-outer -->\n      <ng-container *ngIf="slides.toArray().length">\n        <div class="owl-nav" [ngClass]="{\'disabled\': navData?.disabled}">\n          <div class="owl-prev" [ngClass]="{\'disabled\': navData?.prev?.disabled}" (click)="prev()" [innerHTML]="navData?.prev?.htmlText"></div>\n          <div class="owl-next" [ngClass]="{\'disabled\': navData?.next?.disabled}" (click)="next()" [innerHTML]="navData?.next?.htmlText"></div>\n        </div> <!-- /.owl-nav -->\n        <div class="owl-dots" [ngClass]="{\'disabled\': dotsData?.disabled}">\n          <div *ngFor="let dot of dotsData?.dots" class="owl-dot" [ngClass]="{\'active\': dot.active, \'owl-dot-text\': dot.showInnerContent}" (click)="moveByDot(dot.id)">\n            <span [innerHTML]="dot.innerContent"></span>\n          </div>\n        </div> <!-- /.owl-dots -->\n      </ng-container>\n    </div> <!-- /.owl-carousel owl-loaded -->\n  ',
                    providers: [NavigationService, AutoplayService, CarouselService, LazyLoadService, AnimateService, AutoHeightService, HashService],
                    styles: ['.owl-theme { display: block; }'],
                },
            ],
        },
    ];
    CarouselComponent.ctorParameters = function () {
        return [
            { type: core.ElementRef },
            { type: ResizeService },
            { type: CarouselService },
            { type: NavigationService },
            { type: AutoplayService },
            { type: LazyLoadService },
            { type: AnimateService },
            { type: AutoHeightService },
            { type: HashService },
            { type: OwlLogger },
            { type: core.ChangeDetectorRef },
            { type: undefined, decorators: [{ type: core.Inject, args: [DOCUMENT] }] },
        ];
    };
    CarouselComponent.propDecorators = {
        slides: [{ type: core.ContentChildren, args: [CarouselSlideDirective] }],
        translated: [{ type: core.Output }],
        dragging: [{ type: core.Output }],
        change: [{ type: core.Output }],
        changed: [{ type: core.Output }],
        initialized: [{ type: core.Output }],
        options: [{ type: core.Input }],
        onVisibilityChange: [{ type: core.HostListener, args: ['document:visibilitychange', ['$event']] }],
    };

    var StageComponent = /** @class */ (function () {
        function StageComponent(zone, el, renderer, carouselService, animateService) {
            var _this = this;
            this.zone = zone;
            this.el = el;
            this.renderer = renderer;
            this.carouselService = carouselService;
            this.animateService = animateService;
            /**
             * Object with data needed for dragging
             */
            this._drag = {
                time: null,
                target: null,
                pointer: null,
                stage: {
                    start: null,
                    current: null,
                },
                direction: null,
                active: false,
                moving: false,
            };
            /**
             * Subject for notification when the carousel's rebuilding caused by resize event starts
             */
            this._oneDragMove$ = new rxjs.Subject();
            /**
             * Passes this to _oneMouseTouchMove();
             */
            this.bindOneMouseTouchMove = function (ev) {
                _this._oneMouseTouchMove(ev);
            };
            /**
             * Passes this to _onDragMove();
             */
            this.bindOnDragMove = function (ev) {
                _this._onDragMove(ev);
            };
            /**
             * Passes this to _onDragMove();
             */
            this.bindOnDragEnd = function (ev) {
                // this.zone.run(() => {
                _this._onDragEnd(ev);
                // });
            };
            /**
             * Attaches handler for 'click' event on any element in .owl-stage in order to prevent dragging when moving of cursor is less than 3px
             */
            this._oneClickHandler = function () {
                _this.listenerOneClick = _this.renderer.listen(_this._drag.target, 'click', function () {
                    return false;
                });
                _this.listenerOneClick();
            };
        }
        StageComponent.prototype.onMouseDown = function (event) {
            if (this.owlDraggable.isMouseDragable) {
                this._onDragStart(event);
            }
        };
        StageComponent.prototype.onTouchStart = function (event) {
            if (this.owlDraggable.isTouchDragable) {
                this._onDragStart(event);
            }
        };
        StageComponent.prototype.onTouchCancel = function (event) {
            this._onDragEnd(event);
        };
        StageComponent.prototype.onDragStart = function () {
            if (this.owlDraggable.isMouseDragable) {
                return false;
            }
        };
        StageComponent.prototype.onSelectStart = function () {
            if (this.owlDraggable.isMouseDragable) {
                return false;
            }
        };
        StageComponent.prototype.ngOnInit = function () {
            var _this = this;
            this._oneMoveSubsription = this._oneDragMove$.pipe(operators.first()).subscribe(function () {
                _this._sendChanges();
            });
        };
        StageComponent.prototype.ngOnDestroy = function () {
            this._oneMoveSubsription.unsubscribe();
        };
        /**
         * Handles `touchstart` and `mousedown` events.
         * @todo Horizontal swipe threshold as option
         * @todo #261
         * @param event - The event arguments.
         */
        StageComponent.prototype._onDragStart = function (event) {
            var _this = this;
            var stage = null;
            if (event.which === 3) {
                return;
            }
            stage = this._prepareDragging(event);
            this._drag.time = new Date().getTime();
            this._drag.target = event.target;
            this._drag.stage.start = stage;
            this._drag.stage.current = stage;
            this._drag.pointer = this._pointer(event);
            this.listenerMouseUp = this.renderer.listen(document, 'mouseup', this.bindOnDragEnd);
            this.listenerTouchEnd = this.renderer.listen(document, 'touchend', this.bindOnDragEnd);
            this.zone.runOutsideAngular(function () {
                _this.listenerOneMouseMove = _this.renderer.listen(document, 'mousemove', _this.bindOneMouseTouchMove);
                _this.listenerOneTouchMove = _this.renderer.listen(document, 'touchmove', _this.bindOneMouseTouchMove);
            });
        };
        /**
         * Attaches listeners to `touchmove` and `mousemove` events; initiates updating carousel after starting dragging
         * @param event event objech of mouse or touch event
         */
        StageComponent.prototype._oneMouseTouchMove = function (event) {
            var delta = this._difference(this._drag.pointer, this._pointer(event));
            if (this.listenerATag) {
                this.listenerATag();
            }
            if (Math.abs(delta.x) < 3 && Math.abs(delta.y) < 3 && this._is('valid')) {
                return;
            }
            if (Math.abs(delta.x) < 3 && Math.abs(delta.x) < Math.abs(delta.y) && this._is('valid')) {
                return;
            }
            this.listenerOneMouseMove();
            this.listenerOneTouchMove();
            this._drag.moving = true;
            this.blockClickAnchorInDragging(event);
            this.listenerMouseMove = this.renderer.listen(document, 'mousemove', this.bindOnDragMove);
            this.listenerTouchMove = this.renderer.listen(document, 'touchmove', this.bindOnDragMove);
            event.preventDefault();
            this._enterDragging();
            this._oneDragMove$.next(event);
            // this._sendChanges();
        };
        /**
         * Attaches handler to HTMLAnchorElement for preventing click while carousel is being dragged
         * @param event event object
         */
        StageComponent.prototype.blockClickAnchorInDragging = function (event) {
            var target = event.target;
            while (target && !(target instanceof HTMLAnchorElement)) {
                target = target.parentElement;
            }
            if (target instanceof HTMLAnchorElement) {
                this.listenerATag = this.renderer.listen(target, 'click', function () {
                    return false;
                });
            }
        };
        /**
         * Handles the `touchmove` and `mousemove` events.
         * @todo #261
         * @param event - The event arguments.
         */
        StageComponent.prototype._onDragMove = function (event) {
            var stage;
            var stageOrExit = this.carouselService.defineNewCoordsDrag(event, this._drag);
            if (stageOrExit === false) {
                return;
            }
            stage = stageOrExit;
            event.preventDefault();
            this._drag.stage.current = stage;
            this._animate(stage.x - this._drag.stage.start.x);
        };
        /**
         * Moves .owl-stage left-right
         * @param coordinate coordinate to be set to .owl-stage
         */
        StageComponent.prototype._animate = function (coordinate) {
            this.renderer.setStyle(this.el.nativeElement.children[0], 'transform', 'translate3d(' + coordinate + 'px,0px,0px');
            this.renderer.setStyle(this.el.nativeElement.children[0], 'transition', '0s');
        };
        /**
         * Handles the `touchend` and `mouseup` events.
         * @todo #261
         * @todo Threshold for click event
         * @param event - The event arguments.
         */
        StageComponent.prototype._onDragEnd = function (event) {
            this.carouselService.owlDOMData.isGrab = false;
            this.listenerOneMouseMove();
            this.listenerOneTouchMove();
            if (this._drag.moving) {
                this.renderer.setStyle(this.el.nativeElement.children[0], 'transform', '');
                this.renderer.setStyle(
                    this.el.nativeElement.children[0],
                    'transition',
                    this.carouselService.speed(+this.carouselService.settings.dragEndSpeed || this.carouselService.settings.smartSpeed) / 1000 + 's',
                );
                this._finishDragging(event);
                this.listenerMouseMove();
                this.listenerTouchMove();
            }
            this._drag = {
                time: null,
                target: null,
                pointer: null,
                stage: {
                    start: null,
                    current: null,
                },
                direction: null,
                active: false,
                moving: false,
            };
            // this.carouselService.trigger('dragged');
            this.listenerMouseUp();
            this.listenerTouchEnd();
        };
        /**
         * Prepares data for dragging carousel. It starts after firing `touchstart` and `mousedown` events.
         * @param event - The event arguments.
         * @returns stage - object with 'x' and 'y' coordinates of .owl-stage
         */
        StageComponent.prototype._prepareDragging = function (event) {
            return this.carouselService.prepareDragging(event);
        };
        /**
         * Finishes dragging
         * @param event object event of 'mouseUp' of 'touchend' events
         */
        StageComponent.prototype._finishDragging = function (event) {
            this.carouselService.finishDragging(event, this._drag, this._oneClickHandler);
        };
        /**
         * Gets unified pointer coordinates from event.
         * @param event The `mousedown` or `touchstart` event.
         * @returns Contains `x` and `y` coordinates of current pointer position.
         */
        StageComponent.prototype._pointer = function (event) {
            return this.carouselService.pointer(event);
        };
        /**
         * Gets the difference of two vectors.
         * @param first The first vector.
         * @param second- The second vector.
         * @returns The difference.
         */
        StageComponent.prototype._difference = function (firstC, second) {
            return this.carouselService.difference(firstC, second);
        };
        /**
         * Checks whether the carousel is in a specific state or not.
         * @param specificState The state to check.
         * @returns The flag which indicates if the carousel is busy.
         */
        StageComponent.prototype._is = function (specificState) {
            return this.carouselService.is(specificState);
        };
        /**
         * Enters a state.
         * @param name The state name.
         */
        StageComponent.prototype._enter = function (name) {
            this.carouselService.enter(name);
        };
        /**
         * Sends all data needed for View.
         */
        StageComponent.prototype._sendChanges = function () {
            this.carouselService.sendChanges();
        };
        /**
         * Handler for transitioend event
         */
        StageComponent.prototype.onTransitionEnd = function () {
            this.carouselService.onTransitionEnd();
        };
        /**
         * Enters into a 'dragging' state
         */
        StageComponent.prototype._enterDragging = function () {
            this.carouselService.enterDragging();
        };
        /**
         * Handles the end of 'animationend' event
         * @param id Id of slides
         */
        StageComponent.prototype.clear = function (id) {
            this.animateService.clear(id);
        };
        return StageComponent;
    })();
    StageComponent.decorators = [
        {
            type: core.Component,
            args: [
                {
                    selector: 'owl-stage',
                    template:
                        "\n    <div>\n      <div class=\"owl-stage\" [ngStyle]=\"{'width': stageData.width + 'px',\n                                        'transform': stageData.transform,\n                                        'transition': stageData.transition,\n                                        'padding-left': stageData.paddingL ? stageData.paddingL + 'px' : '',\n                                        'padding-right': stageData.paddingR ? stageData.paddingR + 'px' : '' }\"\n          (transitionend)=\"onTransitionEnd()\">\n        <ng-container *ngFor=\"let slide of slidesData; let i = index\">\n          <div class=\"owl-item\" [ngClass]=\"slide.classes\"\n                                [ngStyle]=\"{'width': slide.width + 'px',\n                                            'margin-left': slide.marginL ? slide.marginL + 'px' : '',\n                                            'margin-right': slide.marginR ? slide.marginR + 'px' : '',\n                                            'left': slide.left}\"\n                                (animationend)=\"clear(slide.id)\"\n                                [@autoHeight]=\"slide.heightState\">\n            <ng-template *ngIf=\"slide.load\" [ngTemplateOutlet]=\"slide.tplRef\"></ng-template>\n          </div><!-- /.owl-item -->\n        </ng-container>\n      </div><!-- /.owl-stage -->\n    </div>\n  ",
                    animations: [
                        animations.trigger('autoHeight', [
                            animations.state('nulled', animations.style({ height: 0 })),
                            animations.state('full', animations.style({ height: '*' })),
                            animations.transition('full => nulled', [
                                // style({height: '*'}),
                                animations.animate('700ms 350ms'),
                            ]),
                            animations.transition('nulled => full', [
                                // style({height: 0}),
                                animations.animate(350),
                            ]),
                        ]),
                    ],
                },
            ],
        },
    ];
    StageComponent.ctorParameters = function () {
        return [{ type: core.NgZone }, { type: core.ElementRef }, { type: core.Renderer2 }, { type: CarouselService }, { type: AnimateService }];
    };
    StageComponent.propDecorators = {
        owlDraggable: [{ type: core.Input }],
        stageData: [{ type: core.Input }],
        slidesData: [{ type: core.Input }],
        onMouseDown: [{ type: core.HostListener, args: ['mousedown', ['$event']] }],
        onTouchStart: [{ type: core.HostListener, args: ['touchstart', ['$event']] }],
        onTouchCancel: [{ type: core.HostListener, args: ['touchcancel', ['$event']] }],
        onDragStart: [{ type: core.HostListener, args: ['dragstart'] }],
        onSelectStart: [{ type: core.HostListener, args: ['selectstart'] }],
    };

    var OwlRouterLinkDirective = /** @class */ (function () {
        function OwlRouterLinkDirective(router, route, tabIndex, renderer, el) {
            this.router = router;
            this.route = route;
            this.stopLink = false;
            this.commands = [];
            if (tabIndex == null) {
                renderer.setAttribute(el.nativeElement, 'tabindex', '0');
            }
        }
        Object.defineProperty(OwlRouterLinkDirective.prototype, 'owlRouterLink', {
            set: function (commands) {
                if (commands != null) {
                    this.commands = Array.isArray(commands) ? commands : [commands];
                } else {
                    this.commands = [];
                }
            },
            enumerable: false,
            configurable: true,
        });
        Object.defineProperty(OwlRouterLinkDirective.prototype, 'preserveQueryParams', {
            /**
             * @deprecated 4.0.0 use `queryParamsHandling` instead.
             */
            set: function (value) {
                if (core.isDevMode() && console && console.warn) {
                    console.warn('preserveQueryParams is deprecated!, use queryParamsHandling instead.');
                }
                this.preserve = value;
            },
            enumerable: false,
            configurable: true,
        });
        OwlRouterLinkDirective.prototype.onClick = function () {
            var extras = {
                skipLocationChange: attrBoolValue(this.skipLocationChange),
                replaceUrl: attrBoolValue(this.replaceUrl),
            };
            if (this.stopLink) {
                return false;
            }
            this.router.navigateByUrl(this.urlTree, extras);
            return true;
        };
        Object.defineProperty(OwlRouterLinkDirective.prototype, 'urlTree', {
            get: function () {
                return this.router.createUrlTree(this.commands, {
                    relativeTo: this.route,
                    queryParams: this.queryParams,
                    fragment: this.fragment,
                    queryParamsHandling: this.queryParamsHandling,
                    preserveFragment: attrBoolValue(this.preserveFragment),
                });
            },
            enumerable: false,
            configurable: true,
        });
        return OwlRouterLinkDirective;
    })();
    OwlRouterLinkDirective.decorators = [{ type: core.Directive, args: [{ selector: ':not(a)[owlRouterLink]' }] }];
    OwlRouterLinkDirective.ctorParameters = function () {
        return [
            { type: router.Router },
            { type: router.ActivatedRoute },
            { type: String, decorators: [{ type: core.Attribute, args: ['tabindex'] }] },
            { type: core.Renderer2 },
            { type: core.ElementRef },
        ];
    };
    OwlRouterLinkDirective.propDecorators = {
        queryParams: [{ type: core.Input }],
        fragment: [{ type: core.Input }],
        queryParamsHandling: [{ type: core.Input }],
        preserveFragment: [{ type: core.Input }],
        skipLocationChange: [{ type: core.Input }],
        replaceUrl: [{ type: core.Input }],
        stopLink: [{ type: core.Input }],
        owlRouterLink: [{ type: core.Input }],
        preserveQueryParams: [{ type: core.Input }],
        onClick: [{ type: core.HostListener, args: ['click'] }],
    };
    /**
     * @description
     *
     * Lets you link to specific routes in your app.
     *
     * See `RouterLink` for more information.
     *
     * @ngModule RouterModule
     *
     * @publicApi
     */
    var OwlRouterLinkWithHrefDirective = /** @class */ (function () {
        function OwlRouterLinkWithHrefDirective(router$1, route, locationStrategy) {
            var _this = this;
            this.router = router$1;
            this.route = route;
            this.locationStrategy = locationStrategy;
            this.stopLink = false;
            this.commands = [];
            this.subscription = router$1.events.subscribe(function (s) {
                if (s instanceof router.NavigationEnd) {
                    _this.updateTargetUrlAndHref();
                }
            });
        }
        Object.defineProperty(OwlRouterLinkWithHrefDirective.prototype, 'owlRouterLink', {
            set: function (commands) {
                if (commands != null) {
                    this.commands = Array.isArray(commands) ? commands : [commands];
                } else {
                    this.commands = [];
                }
            },
            enumerable: false,
            configurable: true,
        });
        Object.defineProperty(OwlRouterLinkWithHrefDirective.prototype, 'preserveQueryParams', {
            set: function (value) {
                if (core.isDevMode() && console && console.warn) {
                    console.warn('preserveQueryParams is deprecated, use queryParamsHandling instead.');
                }
                this.preserve = value;
            },
            enumerable: false,
            configurable: true,
        });
        OwlRouterLinkWithHrefDirective.prototype.ngOnChanges = function (changes) {
            this.updateTargetUrlAndHref();
        };
        OwlRouterLinkWithHrefDirective.prototype.ngOnDestroy = function () {
            this.subscription.unsubscribe();
        };
        OwlRouterLinkWithHrefDirective.prototype.onClick = function (button, ctrlKey, metaKey, shiftKey) {
            if (button !== 0 || ctrlKey || metaKey || shiftKey) {
                return true;
            }
            if (typeof this.target === 'string' && this.target !== '_self') {
                return true;
            }
            if (this.stopLink) {
                return false;
            }
            var extras = {
                skipLocationChange: attrBoolValue(this.skipLocationChange),
                replaceUrl: attrBoolValue(this.replaceUrl),
            };
            this.router.navigateByUrl(this.urlTree, extras);
            return false;
        };
        OwlRouterLinkWithHrefDirective.prototype.updateTargetUrlAndHref = function () {
            this.href = this.locationStrategy.prepareExternalUrl(this.router.serializeUrl(this.urlTree));
        };
        Object.defineProperty(OwlRouterLinkWithHrefDirective.prototype, 'urlTree', {
            get: function () {
                return this.router.createUrlTree(this.commands, {
                    relativeTo: this.route,
                    queryParams: this.queryParams,
                    fragment: this.fragment,
                    queryParamsHandling: this.queryParamsHandling,
                    preserveFragment: attrBoolValue(this.preserveFragment),
                });
            },
            enumerable: false,
            configurable: true,
        });
        return OwlRouterLinkWithHrefDirective;
    })();
    OwlRouterLinkWithHrefDirective.decorators = [{ type: core.Directive, args: [{ selector: 'a[owlRouterLink]' }] }];
    OwlRouterLinkWithHrefDirective.ctorParameters = function () {
        return [{ type: router.Router }, { type: router.ActivatedRoute }, { type: common.LocationStrategy }];
    };
    OwlRouterLinkWithHrefDirective.propDecorators = {
        target: [{ type: core.HostBinding, args: ['attr.target'] }, { type: core.Input }],
        queryParams: [{ type: core.Input }],
        fragment: [{ type: core.Input }],
        queryParamsHandling: [{ type: core.Input }],
        preserveFragment: [{ type: core.Input }],
        skipLocationChange: [{ type: core.Input }],
        replaceUrl: [{ type: core.Input }],
        stopLink: [{ type: core.Input }],
        href: [{ type: core.HostBinding }],
        owlRouterLink: [{ type: core.Input }],
        preserveQueryParams: [{ type: core.Input }],
        onClick: [{ type: core.HostListener, args: ['click', ['$event.button', '$event.ctrlKey', '$event.metaKey', '$event.shiftKey']] }],
    };
    function attrBoolValue(s) {
        return s === '' || !!s;
    }

    var routes = [];
    var CarouselModule = /** @class */ (function () {
        function CarouselModule() {}
        return CarouselModule;
    })();
    CarouselModule.decorators = [
        {
            type: core.NgModule,
            args: [
                {
                    imports: [common.CommonModule],
                    declarations: [CarouselComponent, CarouselSlideDirective, StageComponent, OwlRouterLinkDirective, OwlRouterLinkWithHrefDirective],
                    exports: [CarouselComponent, CarouselSlideDirective, OwlRouterLinkDirective, OwlRouterLinkWithHrefDirective],
                    providers: [WINDOW_PROVIDERS, ResizeService, DOCUMENT_PROVIDERS, OwlLogger],
                },
            ],
        },
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.CarouselComponent = CarouselComponent;
    exports.CarouselModule = CarouselModule;
    exports.CarouselSlideDirective = CarouselSlideDirective;
    exports.OwlRouterLinkDirective = OwlRouterLinkDirective;
    exports.OwlRouterLinkWithHrefDirective = OwlRouterLinkWithHrefDirective;
    exports.SlidesOutputData = SlidesOutputData;
    exports.ɵa = NavigationService;
    exports.ɵb = CarouselService;
    exports.ɵc = OwlLogger;
    exports.ɵd = AutoplayService;
    exports.ɵe = WINDOW;
    exports.ɵf = WindowRef;
    exports.ɵg = BrowserWindowRef;
    exports.ɵh = windowFactory;
    exports.ɵi = browserWindowProvider;
    exports.ɵj = windowProvider;
    exports.ɵk = WINDOW_PROVIDERS;
    exports.ɵl = DOCUMENT;
    exports.ɵm = DocumentRef;
    exports.ɵn = BrowserDocumentRef;
    exports.ɵo = documentFactory;
    exports.ɵp = browserDocumentProvider;
    exports.ɵq = documentProvider;
    exports.ɵr = DOCUMENT_PROVIDERS;
    exports.ɵs = LazyLoadService;
    exports.ɵt = AnimateService;
    exports.ɵu = AutoHeightService;
    exports.ɵv = HashService;
    exports.ɵw = ResizeService;
    exports.ɵx = StageComponent;

    Object.defineProperty(exports, '__esModule', { value: true });
});
//# sourceMappingURL=ngx-owl-carousel-o.umd.js.map
